/***************************************************************************
                          ProgressDialog.cpp  -  description
                             -------------------
    begin                : Die Mai 15 15:34:19 CEST 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// QT Includes
#include <qlayout.h>
#include <qtooltip.h>

// KDE includes
#include <kapplication.h>
#include <kiconloader.h>
#include <klistview.h>
#include <klocale.h>
#include <kpushbutton.h>
#include <krun.h>

// Own includes
#include "ProgressDialog.h"

extern QWidget* launch( QRect rect, const QStringList & list = 0, KRenameImpl* impl = 0 );

ProgressDialog::ProgressDialog( QWidget* parent, const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
    resize( 400, 380 );
    setCaption( i18n("Progress") );
    setIcon( BarIcon( "krename" ) );

    ProgressDialogLayout = new QVBoxLayout( this, 11, 6 );
    Layout1 = new QHBoxLayout( 0, 0, 6 );

    display = new KListView( this );
    display->addColumn( i18n("Messages") );
    display->addColumn( "sort" );
    display->setColumnWidthMode( 0, QListView::Maximum );
    display->setColumnWidthMode( 1, QListView::Manual );
    display->setColumnWidth( 1, 0 );
    display->setSorting( -1 );
    display->setUpdatesEnabled( false );
    
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Minimum );

    buttonClose = new KPushButton( this );
    buttonClose->setText( i18n( "&Close" ) );

    buttonRestart = new KPushButton( this );
    buttonRestart->setText( i18n( "&Rename More..." ) );
    QToolTip::add( buttonRestart, i18n("Restarts KRename") );

    buttonAgain = new KPushButton( this );
    buttonAgain->setText( i18n( "Rename Files &Again...") );
    QToolTip::add( buttonAgain, i18n( "Rename output files again" ) );

    buttonOpenDest = new KPushButton( this );
    buttonOpenDest->setText( i18n("&Open Destination...") );

    Layout1->addItem( spacer );
    Layout1->addWidget( buttonClose );
    Layout1->addWidget( buttonRestart );
    Layout1->addWidget( buttonAgain );
    Layout1->addWidget( buttonOpenDest );
    Layout1->addItem( spacer );

    ProgressDialogLayout->addWidget( display );
    ProgressDialogLayout->addLayout( Layout1 );

    connect( buttonRestart, SIGNAL(clicked()), this, SLOT(restart() ));
    connect( buttonClose, SIGNAL(clicked()), this, SLOT(quitAll()));
    connect( buttonAgain, SIGNAL(clicked()), this ,SLOT(again()));
    connect( buttonOpenDest, SIGNAL(clicked()), this, SLOT(openDest()));

    m_count = 0;
    hide();
}

ProgressDialog::~ProgressDialog()
{
    delete renamedFiles;
}

void ProgressDialog::print( QString text, QString pixmap )
{
    // set an icon if we have none yet
    if( pixmap.isNull() )
        pixmap = "ok";
        
    KListViewItem* item = new KListViewItem( display, text, count() );
    item->setPixmap( 0, SmallIcon(pixmap) );
    display->insertItem( item );
    
    kapp->processEvents( 0 );
}

void ProgressDialog::error( QString text )
{
    KListViewItem* item = new KListViewItem( display, QString( i18n("Error: %1") ).arg(text), count() );
    item->setPixmap( 0, SmallIcon("cancel") );
    display->insertItem( item );

    kapp->processEvents( 0 );
}

void ProgressDialog::warning( QString text )
{
    KListViewItem* item = new KListViewItem( display, QString( i18n("Warning: %1") ).arg(text), count() );
    item->setPixmap( 0, SmallIcon("idea") );
    display->insertItem( item );    

    kapp->processEvents( 0 );
}

void ProgressDialog::quitAll()
{
    KApplication::exit(0);
}

void ProgressDialog::done()
{
    display->setUpdatesEnabled( true );
    display->setSorting( 1, true );
    display->sort();
    display->setSorting( -1 );
    display->triggerUpdate();
    display->ensureItemVisible( display->lastItem() );
    
    kapp->processEvents( 0 );
}

void ProgressDialog::show()
{
    QWidget::show();

    int w = kapp->desktop()->width();
    int h = kapp->desktop()->height();
    move( (w-width())/2,(h-height())/2 );
}

void ProgressDialog::restart()
{
    QWidget* krename = launch( QRect( 0, 0, 0, 0 ) );
    krename->show();
    close();
}

void ProgressDialog::again()
{
    QWidget* krename = launch( QRect( 0, 0, 0, 0 ), *renamedFiles );
    krename->show();
    close();
}

void ProgressDialog::openDest()
{
    if(!m_dest.isEmpty() )
        new KRun( m_dest );
}

QString ProgressDialog::count()
{
    QString s;
    return s.sprintf( "%0*i", 7, ++m_count );
}
