/***************************************************************************
                          batchrenamer.cpp  -  description
                             -------------------
    begin                : Sat Aug 18 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef VERSION
    #define VERSION "unknown"
#endif

// OS includes
#include <stdio.h>
#include <pwd.h>
#include <grp.h>
#include <unistd.h>
// chmod:
#include <sys/types.h>
#include <sys/stat.h>

// QT includes
#include <qdir.h>
#include <qregexp.h>
#include <qprogressdialog.h>

// KDE includes
#include <kapplication.h>
#include <kio/job.h>
#include <klocale.h>

// Own includes
#include "ProgressDialog.h"
#include "batchrenamer.h"
#include "fileoperation.h"
#include "pluginloader.h"
#include "kmylistview.h"

// a is used in find number and
// required for skipping.
// be sure to set it to 0 in the destructor
static int a = 0;

BatchRenamer::BatchRenamer()
{
}

BatchRenamer::~BatchRenamer()
{
    a = 0;
}

void BatchRenamer::processFiles( values* val, ProgressDialog* p, PluginLoader* plug  )
{
    QString tmp;
    t.start();
    a = 0;
    val->batchrenamer = (void*)this;
    for( unsigned int i = 0; i < val->files.count(); i++) {
        if( val->m == RENAME ) // final Path = source Path
            val->files[i].destination_dir = val->files[i].source_dir;
        else
            val->files[i].destination_dir = val->dirname;

        if( i == 0 )
            p->setDestination( val->files[i].destination_dir );

        // Tell plugins that we are renaming the basename
        val->partoffile = BASENAME;
        val->files[i].destination_name = processString( val->text, val->files[i].source_name, val, i, plug );
        
        if( !val->extext.isEmpty() && val->extext != "." ) {
            tmp = val->files[i].extension;

            // Tell plugins that we are renaming the extension
            val->partoffile = EXTENSION;
            tmp = processString( val->extext, tmp, val, i, plug );

            if( !tmp.isEmpty() )
                tmp.prepend( "." );

            val->files[i].destination_name += tmp;
        }

        /*
         * The last step: make changes from
         * the user visible
         * WARNING: same code in createPreview( ... ) and
         * processFiles( ... )
         */
        if( !val->changes.isEmpty() )
            for( unsigned int z = 0; z < val->changes.count(); z++ ) {
                QString file = val->changes[z].filename;
                if( file.right( 1 ) == "/" )
                    file = file.left( file.length() - 1 );

                if( file == (val->files[i].source_dir + val->files[i].source_name + ( val->files[i].extension.isEmpty() ? "" : "." ) + val->files[i].extension) ) {
                    val->files[i].destination_name = val->changes[z].user;
                }
            }

        // Assemble filenames
        parseSubdirs( &val->files[i] );
        val->files[i].source_name = val->files[i].source_dir + val->files[i].source_name + ( val->files[i].extension.isEmpty() ? "" : "." ) + val->files[i].extension;
        val->files[i].destination_name = val->files[i].destination_dir + val->files[i].destination_name;

        /*
         * take care of renamed directories and
         * correct the paths of their contents
         */
        if( val->m == RENAME || val->m == MOVE ) {
            QFileInfo info( val->files[i].source_name );
            if( info.isDir() )
                for( unsigned int c = i; c < val->files.count(); c++ ) {
                    if( val->files[c].source_dir.left( val->files[i].source_name.length() + 1 ) == ( val->files[i].source_name + "/" ) ) {
                        val->files[c].source_dir.replace( 0, val->files[i].source_name.length(), val->files[i].destination_name );
                    }
                    //source_name=test01
                    //source_dir= /home/dominik/
                }
        }
    }
    a = 0;

    p->print( QString( i18n("Filenames Processed after %1 seconds.")).arg(t.elapsed()/1000) );

    work( val, p, plug );
}

QString BatchRenamer::processString( QString text, QString oldname, values* val, int i, PluginLoader* plug )
{
    /*
     * Come on! Grep for this text and help me!
     *
     * note about krename escape sequences
     * for certain characters:
     *
     * Krename will have problems with files
     * which contain one of the following
     * unicode characters: 60000, 60001, 60002
     * 60003, 60004, 60005, 60006.
     *
     * This is not a good solution, if you have a
     * better one please tell me about it!
     */

    oldname = doEscape( oldname );

    /*
     * Call here all functions that handle
     * arguments that are single tokens (&,%,...).
     * or in [brackets]
     */
    text = findBrackets( oldname, text, val, i, plug );
    text = findAndProcess( "$", text, oldname );
    text = findAndProcess( "%", text, oldname.lower() );
    text = findAndProcess( "&", text, oldname.upper() );
    text = findAndProcess( "\\", text, oldname.stripWhiteSpace() );
    text = findParagraph( oldname, text );
    text = findStar( oldname, text );
    text = findNumbers( text, val->index, val->files.count(), i, val->skip, val->step );

    /*
     * text is used as argument token for plugins!
     */
    parsePlugins( plug, val, i, text, TYPE_TOKEN );
   
    /*
     * Replace after Plugins !
     * Replace shoud be the last the
     * before re-escaping tokens !
     */
    text = findReplace( text, val->replace );

    // convert special chars back (e.g. &,$)
    text = unEscape( text );

    return text;
}

QString BatchRenamer::parsePlugins( PluginLoader* plug, values* val, int i, QString text, int type )
{
    QPtrListIterator<PluginLoader::PluginLibrary> it( plug->libs );
    QString ret = text;

    if( type == TYPE_FINAL_FILE )
        ret = "";
        
    for( ; it.current(); ++it ) {
        if( (*it)->usePlugin && ((*it)->plugin->type() == type ||
            (*it)->plugin->type() == TYPE_ALL ) )
            ret = doEscape( (*it)->plugin->processFile( val, i, text, type ) );
    }

    return ret;
}

void BatchRenamer::createPreview( QListView* list, values* val, PluginLoader* plug  )
{
    KMyListViewItem* item1 = NULL;
    QString tmp;
    val->batchrenamer = (void*)this;
    a = 0;
    for( unsigned int i = 0; i < val->files.count(); i++) {
        val->files[i].destination_name = processString( val->text, val->files[i].source_name, val, i, plug );

        if( !val->extext.isEmpty() ) {
            tmp = val->files[i].extension;
            tmp = processString( val->extext, tmp, val, i, plug );
            if( !tmp.isEmpty() )
                tmp.prepend( "." );

            val->files[i].destination_name += tmp;
        }

        /*
         * The last step: make changes of
         * the user visible
         * WARNING: same code in createPreview( ... ) and
         * processFiles( ... )
         */
        bool modified = false;

        if( !val->changes.isEmpty() )
            for( unsigned int z = 0; z < val->changes.count(); z++ ) {
                QString file = val->changes[z].filename;
                if( file.right( 1 ) == "/" )
                    file = file.left( file.length() - 1 );
                    
                if( file == (val->files[i].source_dir + val->files[i].source_name + ( val->files[i].extension.isEmpty() ? "" : "." ) + val->files[i].extension) ) {
                    val->files[i].destination_name = val->changes[z].user;
                    modified = true;
                }
            }
        
        QString sname = val->files[i].source_name + ( val->files[i].extension.isEmpty() ? "" : "." ) + val->files[i].extension;
        if( item1 )
            item1 = new KMyListViewItem( modified, list, item1, sname, val->files[i].destination_name );
        else
            // this is the first item in the listview
            item1 = new KMyListViewItem( modified, list, sname, val->files[i].destination_name );
    }
}

void BatchRenamer::work( values* val, ProgressDialog* p, PluginLoader* plug )
{
    FileOperation fop;
    QFile* fundo ( NULL );
    QTextStream* tundo ( NULL );

    if( val->undo ) {
        // Create header for undo script
        fundo = new QFile( val->undoScript );
        if( fundo->open( IO_WriteOnly ) ) {
            tundo = new QTextStream( fundo );
            writeUndoScript( tundo );
        } else {
            val->undo = false;
            p->error( i18n("Can't create undo script :") + fundo->name() );
            delete fundo;
        }
    }

    int error = 0;
    QStringList* renamedFiles = new QStringList();

    progress = new QProgressDialog( QString( i18n("Renaming %1 files.")).arg(val->files.count()), i18n("&Cancel"), val->files.count()+1, p, "progress", TRUE );
    progress->setMinimumDuration( 0 );
    progress->setProgress( 0 );

    /*
     * Give the user some information...
     */
    if( val->m == COPY)
        p->print( QString( i18n("Files will be copied to: %1") ).arg(val->files[0].destination_dir) );
    else if( val->m == MOVE )
        p->print( QString( i18n("Files will be moved to: %1") ).arg(val->files[0].destination_dir) );
    else if( val->m == RENAME )
        p->print( i18n("Input files will be renamed.") );
    else if( val->m == LINK )
        p->print( QString( i18n("Creating symbolic links in: %1") ).arg(val->files[0].destination_dir) );

    unsigned int i;
    for( i = 0; i < val->files.count(); i++) {
        progress->setProgress( i+1 );
        kapp->processEvents( 0 );

        if ( progress->wasCancelled() ) {
            p->warning( i18n("User pressed cancel!") );
            p->warning( i18n("Aborting...") );
            break;
        }

        QCString src = QFile::encodeName( val->files[i].source_name );
        QCString dst = QFile::encodeName( val->files[i].destination_name );
        
        if( QFile::exists( val->files[i].destination_name ) )
            if( val->overwrite)
                p->warning( QString( i18n("Overwriting file %1.") ).arg( dst ));
            else {
                p->error( QString( i18n("Output file %1 exists already.") ).arg( dst ));
                error++;
                renamedFiles->append( val->files[i].destination_name );
                continue;
            }

        if( val->m == RENAME || val->m == MOVE ) {
            if( rename( src, dst ) ) {
                p->error( QString( i18n("Can't rename %1 to %2.") ).arg( src ).arg( dst ));
                error++;
            }
        } else if( val->m == COPY ) {
            if(!fop.fcopy( val->files[i].source_name, val->files[i].destination_name ) )
                error++;
        } else if( val->m == LINK ) {
            if(!FileOperation::slink( src, dst, val->overwrite, p ) )
                error++;
        }

        renamedFiles->append( val->files[i].destination_name );

        /*
         * The renamed file should be on its correct location now,
         * so that we can call the last plugins (e.g. for changing permissions)
         *
         * Remember, the token argument is the filename for this type of plugins!
         *
         * If the return value is not empty an error has occured!
         * The plugin should return an error message in this case!
         */
        QString eplug = parsePlugins( plug, val, i, dst, TYPE_FINAL_FILE );
        if( !eplug.isEmpty() ) {
            p->error( eplug );
            error++;
        }

        /* Create the undo script now */
        if( val->undo ) {
            // Plugins ???
            (*tundo) << "echo \"" << FileOperation::getName( val->files[i].source_name )
                     << " -> " << FileOperation::getName( val->files[i].destination_name ) << "\"" << endl;
            (*tundo) << "mv --force -b --suffix=.krename_ \"" << val->files[i].destination_name
                  << "\" \"" << val->files[i].source_name << "\"" << endl;

        }
    }

    QPtrListIterator<PluginLoader::PluginLibrary> it( plug->libs );
    for( ; it.current(); ++it ) {
        if( (*it)->usePlugin )
            (*it)->plugin->finished();
    }

    const QString m = QString( i18n("Renamed %1 files sucessfully.") ).arg(i-error);
    ( i - error ) ? p->print( m ) : p->warning( m );
    if( i - error > 0 ) p->warning( QString( i18n("%2 errors ocurred!") ).arg(error));
    p->print( QString( i18n("Elapsed time: %1 seconds") ).arg( t.elapsed()/1000 ), "kalarm" );
    p->print( i18n("KRename finsihed the renaming process."), "krename" );
    p->print( i18n("Press close to quit!") );
    p->setRenamedFiles( renamedFiles );

    if( val->undo ) {
        (*tundo) << endl << "echo \"Finished undoing " << val->files.count() << " actions.\"" << endl;
        delete tundo;
        fundo->close();

        // Make fundo exuteable
        if( chmod( (const char*)val->undoScript, (unsigned int) S_IRUSR | S_IWUSR | S_IXUSR ) )
            p->error( i18n("Can't set executable bit on undo script.") );
        delete fundo;
    }

    p->done();
    delete progress;
    val->files.clear();
    delete val;
    delete this;
}

QString BatchRenamer::escape( QString text, QString token, QString sequence )
{
    /*
     * NEVER, ABSOLUTELY NEVER change pos = 0
     * to pos = -1, it won't work !
     * This bug took hours to find and was
     * a serious bug in 1.7.
     */

    int pos = 0;
    do {
       pos = text.find( token, pos );
       if( pos >= 0 )
           text.replace( pos, 1, sequence );
    } while ( pos >= 0 );
    return text;
}

QString BatchRenamer::doEscape( QString text )
{
    text = BatchRenamer::escape( text, "&", QChar( 60000 ) );
    text = BatchRenamer::escape( text, "$", QChar( 60001 ) );
    text = BatchRenamer::escape( text, "%", QChar( 60002 ) );
    text = BatchRenamer::escape( text, "", QChar( 60003 ) );
    text = BatchRenamer::escape( text, "#", QChar( 60004 ) );
    text = BatchRenamer::escape( text, "[", QChar( 60005 ) );
    text = BatchRenamer::escape( text, "]", QChar( 60006 ) );
    text = BatchRenamer::escape( text, "\\", QChar( 60007 ) );
    text = BatchRenamer::escape( text, "/", QChar( 60008 ) );

    return text;
}

QString BatchRenamer::unEscape( QString text )
{
    text = BatchRenamer::escape( text, QChar( 60000 ), "&" );
    text = BatchRenamer::escape( text, QChar( 60001 ), "$" );
    text = BatchRenamer::escape( text, QChar( 60002 ), "%" );
    text = BatchRenamer::escape( text, QChar( 60003 ), "" );
    text = BatchRenamer::escape( text, QChar( 60004 ), "#" );
    text = BatchRenamer::escape( text, QChar( 60005 ), "[" );
    text = BatchRenamer::escape( text, QChar( 60006 ), "]" );
    text = BatchRenamer::escape( text, QChar( 60007 ), "\\" );
    text = BatchRenamer::escape( text, QChar( 60008 ), "/" );
    
    return text;
}

int BatchRenamer::getCharacters( int n )
{
    QString s;
    s.sprintf( "%i", n );
    return s.length();
}

QString BatchRenamer::findAndProcess( QString token, QString text, QString replace )
{
    /*
     * pos can here be -1 because
     * findRev is called with it as a
     * value !
     */
    int pos = -1;
    do {
        pos = text.findRev( token, pos );
        if( pos >= 0 )
            text.replace( pos, 1, replace );
    } while( pos >= 0 );
    return text;
}

QString BatchRenamer::findNumbers( QString text, int index, int count, int i, QValueList<int> skip, int step )
{
    // Rewritten in Version 0.8
    // Added numbers skipping in 1.3
    // Changed again in Version 1.8 to optimize it and fix a bug with skipping numbers
    int pos = 0, counter = 1;

    if( text.contains( "#", FALSE ) <= 0 )
        return text;


    pos = text.find("#", pos);
    pos++;
    while( text[pos] == '#' ) {
        text.remove(pos, 1);
        counter++;
    }

    pos = text.find("#", 0);
    // a is static
    int v = index + (i*step) + a;
    for( unsigned int z = 0; z < skip.count(); z++ ) {
        if( skip[z] == v ) {
            a += step;
            v += step;
            // v = index + (i*step) + a;
        }
    }

    QString temp;
    temp.sprintf("%0*i", counter, v );
    text.replace( pos, 1, temp);

    return findNumbers( text, index, count, i, skip, step );
}

QString BatchRenamer::findParagraph ( QString oldname, QString text )
{
    int pos = -1;
    do {
        pos = text.findRev("", pos);
        if( pos >= 0 ) {
            text.replace( pos, 1, oldname.lower());
            text.replace( pos, 1, oldname[0].upper());
        }
    } while( pos >= 0 );
    return text;
}

QString BatchRenamer::findStar( QString oldname, QString text )
{
    int pos = -1;
    do {
        pos = text.findRev("*", pos);
        if( pos >= 0 ) {
            QString tmp = oldname.lower();
            if( tmp[0].isLetter() )
                tmp[0] = tmp[0].upper();

            for( unsigned int i = 0; i < tmp.length(); i++ )
                if( tmp[i+1].isLetter() && !tmp[i].isLetter() )
                    tmp[i+1] = tmp[i+1].upper();

            text.replace( pos, 1, tmp);
        }
    } while( pos >= 0 );
    return text;
}

QString BatchRenamer::findBrackets( QString oldname, QString text, values* val, int i, PluginLoader* plug )
{
    /*
     * looks for a statement in brackets [ ]
     * and calls findToken() with this statement.
     */

    int num, pos = -1, a;
    QString token;

    if( text.contains("]", FALSE) <= 0 || text.isEmpty() )
        return text;

    num = text.contains("[", FALSE);
    if(num <= 0 )
        return text;

    pos = text.findRev("[", pos);
    a = text.find("]", pos );
    if( a < 0 && pos >= 0 )
        return text;

    if( pos < 0 && a >= 0 )
        return text;

     if( pos >= 0 && a >= 0 ) {
        token = text.mid( pos+1, (a-pos)-1 );
        text.remove( pos, (a-pos)+1 );
        text.insert( pos, findToken( oldname, token, val, i, plug ));
    }
    return findBrackets( oldname, text, val, i, plug );
}

QString BatchRenamer::processToken( QString token, QString oldname, values* val, int i, PluginLoader* plug )
{
    QString tmp;

    /*
     * Call here all functions that handle
     * arguments in brackets.
     */
    tmp = findDate( token );
    if( !tmp.isEmpty() )
        return tmp;

    tmp = findPartStrings( oldname, token );
    if( !tmp.isEmpty() )
        return tmp;

    tmp = findDirName( token, val->files[i].source_dir );
    if( !tmp.isEmpty() )
        return tmp;

    tmp = parsePlugins( plug, val, i, token, TYPE_BRACKET );
    if( !tmp.isNull() ) {
        return tmp;
    } else {
        /*
         * Maybe I should remove this!
         * Krename simply ignores unknown tokens!
         * Usefull for the MP3 Plugin!
         */
        return QString::null;
    }
    
    return token;        
}

QString BatchRenamer::findToken( QString oldname, QString token, values* val, int i, PluginLoader* plug )
{
    enum conversion { LOWER, UPPER, MIXED, STAR, STRIP, NONE, EMPTY, NUMBER };
    unsigned int numwidth = 0;
    
    conversion c = EMPTY;
    if( !token.left(1).compare("$") )
        c = NONE;
    else if( !token.left(1).compare("%") )
        c = LOWER;
    else if( !token.left(1).compare("&") )
        c = UPPER;
    else if( !token.left(1).compare("") )
        c = MIXED;
    else if( !token.left(1).compare("*") )
        c = STAR;
    else if( !token.left(1).compare("\\") )
        c = STRIP;
    else if( !token.left(1).compare("#") ) {
        while( !token.left(1).compare("#") ) {
            token.remove( 0, 1 );
            ++numwidth;
        }
        
        c = NUMBER;
    }

    if( c != EMPTY && c != NUMBER )
        token.remove( 0, 1 );
    
    QString save = token;
    token = processToken( token, oldname, val, i, plug );

    switch( c ) {
        case LOWER:
            token = token.lower();
            break;
        case UPPER:
            token = token.upper();
            break;
        case MIXED:
            token = token.lower();
            token.replace( 0, 1, token[0].upper());
            break;
        case STAR:
            token = findStar( token, "*" );
            break;
        case STRIP:
            token = token.stripWhiteSpace();
            break;
        case NUMBER:
            {
                bool b = false;
                int n = token.toInt( &b );
                if( b )
                    token = token.sprintf("%0*i", numwidth, n );
            }
            break;
        default:
            break;
    }

    /* the token has not been modified
     * so we do not know what to do with it
     * and do not use it.
     */
    if( token == save )
        return QString::null;
        
    return doEscape( token );
}

QString BatchRenamer::findPartStrings( QString oldname, QString token )
{
    QString first, second;
    int pos = -1;

    if( token.contains("-") ) {
        pos = token.find( "-", 0 );
        first = token.left( pos );
        // ------- Code OK ^ !

        second = token.mid( pos+1, token.length() );

        // version < 1.7
        // return oldname.mid( first.toInt()-1, second.toInt()-first.toInt() +1 );
        // version > 1.7
        //return oldname.mid( first.toInt()-1, second.toInt()-first.toInt() );
        // version > 1.8

        bool ok;
        int sec = second.toInt( &ok );
        if( !ok || sec == 0 )
            sec = oldname.length();

        /*
         * x should not be larger than the old name
         * and not smaller than zero.
         */
        int x = sec-first.toInt();
        if( x > (signed int)oldname.length() || x < 0 )
            x = oldname.length()-first.toInt();

        /*
         * if I would comment my code I would understand this line :)
         * without this line, there is sometimes the last letter
         * of a filename missing.
         */
        if( x != -1 )
            x++;

        return oldname.mid( first.toInt()-1, x );
    } else if( token.contains(";") ) {
        pos = token.find( ";", 0 );

        first = token.left( pos );
        second = token.mid( pos+1, token.length() );

        return oldname.mid( first.toInt()-1, second.toInt() );
    } else {
        bool ok = false;
        int number = token.toInt( &ok );
        
        if( ok && (number <= (signed int)oldname.length() && number > 0 ) ) 
            return QString(oldname[ number -1 ]);
        else
            return QString::null;
    }
}

QString BatchRenamer::findDirName( QString token, QString path )
{
    if( token.left( 7 ).lower() == "dirname" ) {
        if( path.right( 1 ) == "/" )
            path = path.left( path.length() - 1);

        int recursion = 1;
        if( token.length() > 7 ) {
            token = token.right( token.length() - 7 );
            recursion = token.contains( "." );
            if(  recursion != (signed int)token.length() )
                return QString::null;

            recursion++;
        }

        return path.section( "/", recursion * -1, recursion * -1);
    }

    return QString::null;
}

QString BatchRenamer::findDate( QString token )
{
    QDate d = QDate::currentDate();
    QTime t = QTime::currentTime();
    QString tmp, text;

    token = token.lower();
    if( token == "date" )
        return tmp.sprintf("%0*i-%0*i-%i", 2, d.day(), 2, d.month(), d.year() );
    else if( token == "year" )
        return QString( "%1" ).arg( d.year() );
    else if( token == "month" )
        return tmp.sprintf("%0*i", 2, d.month() );
    else if( token == "day" )
        return tmp.sprintf("%0*i", 2, d.day() );
    else if( token == "time" )
        return QString( "%1-%2-%3" ).arg( t.hour() ).arg( tmp.sprintf("%0*i", 2, t.minute() ) ).arg( tmp.sprintf("%0*i", 2, t.second() ) );
    else if( token == "hour" )
        return tmp.sprintf("%0*i", 2, t.hour() );
    else if( token == "minute" )
        return tmp.sprintf("%0*i", 2, t.minute() );
    else if( token == "second" )
        return tmp.sprintf("%0*i", 2, t.second() );

    return QString::null;
}

QString BatchRenamer::findReplace( QString text, QValueList<replacestrings> r )
{
    // Call for each element in replace strings doReplace with correct values
    for( unsigned int i = 0; i < r.count(); i++ ) {
        replacestrings s = r[i];
        text = doReplace( text, doEscape( s.find ), s.replace, s.reg );
    }

    return text;
}

QString BatchRenamer::doReplace( QString text, QString find, QString replace, bool reg )
{
    int pos = 0;
    if( !reg ) {
        do {
            pos = text.find( find, pos );
            if( pos >= 0 ) {
                text.replace( pos, find.length(), replace );
                pos += replace.length();
            }
        } while( pos >= 0 );
    } else {
        // Test this code more!
        pos = 0;
        do {
            QRegExp exp( find );
            pos = exp.search( text, pos );
            if( pos >= 0 ) {
                text.replace( pos, exp.matchedLength(), replace );
                pos += replace.length();
            }
        } while( pos >= 0 );
    }
    return text;
}

void BatchRenamer::writeUndoScript( QTextStream* t )
{
    // write header comments
    (*t) << "#!/bin/bash" << endl
         << "# KRename Undo Script" << endl << "#" << endl
         << "# KRename was written by:" << endl
         << "# Dominik Seichter <domseichter@web.de>" << endl
         << "# http://krename.sourceforge.net" << endl << "#" << endl
         << "# Script generated by KRename Version: " << VERSION << endl << endl
         << "# This script must be started with the option --krename to work!" << endl;

    // write functions:
    (*t) << "echo \"KRename Undo Script\"" << endl
         << "echo \"http://krename.sourceforge.net\"" << endl
         << "echo \"\"" << endl;

    (*t) << "if test --krename = $1 ; then" << endl
         << "   echo \"\"" << endl
         << "else" << endl
         << "   echo \"You have to start this script\"" << endl
         << "   echo \"with the commandline option\"" << endl
         << "   echo \"--krename\"" << endl
         << "   echo \"to undo a rename operation.\"" << endl
         << "   exit" << endl
         << "fi" << endl;
}

void BatchRenamer::parseSubdirs( data* f )
{
    int pos = 0;
    if( (pos = f->destination_name.findRev( "/", -1 ) ) > 0 ) {
        QString dirs = f->destination_name.left( pos );
        f->destination_name = f->destination_name.right( f->destination_name.length() - pos - 1 );    
        f->destination_dir += ( f->destination_dir.right( 1 ) == "/" ) ? "" : "/";
        
        QDir dir( f->destination_dir );
        
        // create the missing subdir now
        int i = 0;
        QString d = "";
        while( !dirs.section( "/", i, i, QString::SectionSkipEmpty ).isEmpty() ) {
            d = dirs.section( "/", i, i, QString::SectionSkipEmpty );
            dir.mkdir( f->destination_dir + d );
            f->destination_dir.append( d + "/" );
            i++;
        }
    }
}

