/***************************************************************************
                          batchrenamer.h  -  description
                             -------------------
    begin                : Sat Aug 18 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BATCHRENAMER_H
#define BATCHRENAMER_H

#include <qdatetime.h>
#include <qvaluelist.h>

class QFile;
class QProgressDialog;
class QString;

/* How many diferrent users and groups
 * KRename supports. Values over 1000
 * make KRename slow, but it may be
 * necessary on bigger systems to
 * increase this value.
 * MAXENTRIES must be < sizeof(int)
 */
#define MAXENTRIES 1000

/* May Value for SpinBoxes
 *
 */
#define SPINMAX 100000

enum mode {
    COPY, MOVE, RENAME, LINK, PREVIEW
};

enum enumpart {         // Useful for plugins that want to what part of the file is
                        // renamed at the moment:
    BASENAME,           // basename is renamed
    EXTENSION           // extension is renamed
};

typedef struct data {
    QString source_name;        // Name of source file
    QString source_dir;         // Path of source file

    QString extension;          // Extension of source file

    QString destination_name;   // Name of output file
    QString destination_dir;    // Path of output file
};

/*
 * Changes made by hand by the user
 * in the preview list view are
 * stored here.
 */
typedef struct manualchanges {
    QString filename;  // input filename
    QString user;      // name the user wants
};

typedef struct replacestrings {
    QString find;      // Text to replace
    QString replace;   // Replace with
    bool reg;          // is it a reg expression ?
};

typedef struct values {
    QString text;           // template
    QString dirname;        // destination dir
    QString extext;         // Extension template
    QString undoScript;     // Filename of undoscript

    int index;              // index for numbers
    int step;               // step for numbers
    bool overwrite;         // overwrite existing files
    bool undo;              // create an undo script

    QValueList<int> skip;   // Numbers to skip
    QValueList<replacestrings> replace; // Repalce strings

    QValueList<data> files;

    QValueList<manualchanges> changes;  // User made changes

    enum mode m;            // in which mode is KRename ?
                            // rename, copy, move, .. preview, ...

    enum enumpart partoffile;

    void* batchrenamer;     // must be casted (BatchRenamer*)
};


/**
  *@author Dominik Seichter
  */

class QListView;
class ProgressDialog;
class PluginLoader;
class BatchRenamer {
    public:
        BatchRenamer();
        ~BatchRenamer();
        void processFiles( values* val, ProgressDialog* p, PluginLoader* plug );
        void createPreview( QListView* list, values* val, PluginLoader* plug );

        // Since 2.1 public, because plugins may want to access them to:
        QString findAndProcess( QString token, QString text, QString replace );

        QString findNumbers( QString text, int index, int count, int i, QValueList<int> skip, int step );
        QString findParagraph( QString oldname, QString text );
        QString findStar( QString oldname, QString text );
        QString findBrackets( QString oldname, QString text, values* val, int i, PluginLoader* plug );
        QString findToken( QString oldname, QString token, values* val, int i, PluginLoader* plug );
        QString processToken( QString token, QString oldname, values* val, int i, PluginLoader* plug );
        QString findPartStrings( QString oldname, QString token );
        QString findDate( QString token );
        static QString findDirName( QString token, QString path );
        QString findReplace( QString text, QValueList<replacestrings> r );  // text is here already the new filename !
        QString doReplace( QString text, QString find, QString replace, bool reg ); // text is here already the new filename !

        static QString doEscape( QString text );
        static QString unEscape( QString text );
        static QString escape( QString text, QString token, QString sequence );

    private:
        int getCharacters( int n ) ;
        void work( values* val, ProgressDialog* p, PluginLoader* plug );
        QString processString( QString text, QString oldname, values* val, int i, PluginLoader* plug );
        void writeUndoScript( QTextStream* t );
        void parseSubdirs( data* f );
        QString parsePlugins( PluginLoader* plug, values* val, int i, QString text, int type );
        
    protected:
        QFile* f;
        QTime t;
        QProgressDialog* progress;
};

#endif
