/***************************************************************************
                          confdialog.cpp  -  description
                             -------------------
    begin                : Sun Jan 27 2002
    copyright            : (C) 2002 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Own includes
#include "confdialog.h"
#include "servicemenuhandler.h"

// QT includes
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qradiobutton.h>
#include <qtooltip.h>

// KDE includes
#include <kiconloader.h>
#include <klocale.h>

ConfDialog::ConfDialog( QWidget* parent, const char* name )
    :  KDialogBase( KDialogBase::IconList, "KRename",
       KDialogBase::Ok | KDialogBase::Cancel | KDialogBase::Default, KDialogBase::Ok, parent, name, true, true ),
       GUIModeSelector()
{
    smh = new ServicemenuHandler( this, "smh" );

    setupTab1();
    setupTab2();
    setupTab3();

    connect( this, SIGNAL( defaultClicked() ), this, SLOT( defaults() ) );
}

ConfDialog::~ConfDialog()
{
}

void ConfDialog::setupTab1()
{
    const QString caption = i18n("Look and Feel");
    QFrame* box = addPage( caption, caption, BarIcon("looknfeel") );
    QVBoxLayout* layout = new QVBoxLayout( box );
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );

    QButtonGroup* group = new QButtonGroup( box );
    group->setColumnLayout(0, Qt::Vertical );
    QVBoxLayout* lgroup = new QVBoxLayout( group->layout() );

    optionWizard = new QRadioButton( group );
    optionWizard->setText( i18n("Use &wizard style GUI (beginners)") );
    optionTabs = new QRadioButton( group );
    optionTabs->setText( i18n("Use &tabbed GUI (advanced users)") );

    lgroup->addWidget( new QLabel( i18n("Configure the look and feel of the KRename GUI:<br>"), group ) );
    lgroup->addWidget( optionWizard );
    lgroup->addWidget( optionTabs );
    lgroup->addItem( spacer );
        
    layout->addWidget( group );
    layout->addItem( spacer );        
}

void ConfDialog::setupTab2()
{
    const QString caption = i18n("KRename");
    QFrame* box = addPage( caption, caption, BarIcon("krename") );
    QVBoxLayout* layout = new QVBoxLayout( box );
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );

    checkContext = new QCheckBox( i18n("&Add KRename to konquerors context menu"), box );     
    checkAsk = new QCheckBox( i18n("Ask &for confirmation on exit"), box );
    checkDir = new QCheckBox( i18n("Add directory names &with filenames"), box );
    checkHiddenDir = new QCheckBox( i18n("Add &hidden directories"), box );
    checkPlugins = new QCheckBox( i18n("&Load file plugins on start"), box );
    checkAutosize = new QCheckBox( i18n("Auto &resize columns in preview"), box );
    
    spinSize = new KIntNumInput( box );
    spinSize->setRange( 20, 500, 1, false );
    spinSize->setLabel( i18n( "&Thumbnail size:" ), AlignLeft | AlignVCenter );

    spinHistory = new KIntNumInput( box );
    spinHistory->setRange( 2, 500, 1, false );
    spinHistory->setLabel( i18n( "&Number of template history items:" ), AlignLeft | AlignVCenter );
    
    layout->addWidget( checkContext );
    layout->addWidget( checkAsk );
    layout->addWidget( checkDir );
    layout->addWidget( checkHiddenDir );
    layout->addWidget( checkPlugins );
    layout->addWidget( checkAutosize );
    layout->addItem( spacer );
    layout->addWidget( spinSize );
    layout->addWidget( spinHistory );
    layout->addItem( spacer );

    checkContext->setChecked( smh->serviceEntryInstalled() );
    
    connect( checkContext, SIGNAL( toggled(bool) ), smh, SLOT( changeService(bool) ));
    connect( smh, SIGNAL( status(bool) ), checkContext, SLOT( setChecked(bool) ) );

    QToolTip::add( checkHiddenDir, i18n("If not checked, KRename will ignore directories starting with a dot during recursive adding.") );
    QToolTip::add( checkPlugins, i18n("Disabling this option increases KRename's startup time, because no KFilePlugins are loaded.") );
}

void ConfDialog::setupTab3()
{
    const QString caption = i18n("File Extension");
    QFrame* box = addPage( caption, caption, BarIcon("unknown") );

    QVBoxLayout* layout = new QVBoxLayout( box );
    QButtonGroup* group = new QButtonGroup( box );
    group->setColumnLayout(0, Qt::Vertical );
    QGridLayout* lgroup = new QGridLayout( group->layout() );
    
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    
    optionFirstDot = new QRadioButton( group );
    optionFirstDot->setText( i18n( "&First dot" ) );

    optionLastDot = new QRadioButton( group );
    optionLastDot->setText( i18n( "&Last dot" ) );

    optionCustomDot = new QRadioButton( group );
    optionCustomDot->setText( i18n( "&Custom dot" ) );

    spinDot = new KIntNumInput( group );
    spinDot->setRange( 1, 100, 1, false );
    spinDot->setEnabled( optionCustomDot->isChecked() );

    lgroup->addWidget( new QLabel( i18n( "File extension starts at...<br>" ), group ), 0, 0 );
    lgroup->addWidget( optionFirstDot, 1, 0 );
    lgroup->addWidget( optionLastDot, 2, 0 );
    lgroup->addWidget( optionCustomDot, 3, 0 );
    lgroup->addWidget( spinDot, 3, 1 );
    
    layout->addWidget( group );
    layout->addItem( spacer );

    QToolTip::add( optionFirstDot, i18n("If the filename is file.tar.bz2, .tar.bz2 is used as file extension." ));
    QToolTip::add( optionLastDot, i18n("If the filename is file.tar.bz2, .bz2 is used as file extension." ));
    QToolTip::add( optionCustomDot, i18n("The file extension starts at the given dot." ));
    QToolTip::add( spinDot, i18n("The file extension starts at the given dot." ));

    connect( optionFirstDot, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( optionLastDot, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( optionCustomDot, SIGNAL( clicked()), this, SLOT( enableControls() ));
}

void ConfDialog::enableControls()
{
    spinDot->setEnabled( optionCustomDot->isChecked() );
}

void ConfDialog::defaults()
{
    optionFirstDot->setChecked( true );
    spinDot->setValue( 1 );

    checkAsk->setChecked( true );
    checkDir->setChecked( false );
    checkHiddenDir->setChecked( false );
    checkPlugins->setChecked( true );
    checkAutosize->setChecked( false );

    optionWizard->setChecked( true );
    
    spinSize->setValue( 80 );

    enableControls();
}

