/***************************************************************************
                          fileoperation.cpp  -  description
                             -------------------
    begin                : Sun Nov 11 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// KDE includes
#include <kapplication.h>
#include <klocale.h>

// QT includes
#include <qfileinfo.h>

// OS includes
#include <stdio.h>
#include <unistd.h>

// Own includes
#include "fileoperation.h"
#include "ProgressDialog.h"

FileOperation::FileOperation()
{
    canceled = false;
}

FileOperation::~FileOperation() { }

bool FileOperation::slink( const char* src, const char* dest, bool overwrite, ProgressDialog* p )
{
    if( QFile::exists( dest ) && overwrite )
        if( remove( dest ))
            p->error( QString( i18n("Can't overwrite %1.") ).arg(dest) );

    if( symlink( src, dest) ) {
        p->error( QString( i18n("Can't create symlink %1.") ).arg(dest) );
        return false;
    }

    return true;
}

bool FileOperation::fcopy( QString src, QString dest )
{

/* ---------------------------- */
    /*
     * KDE way:
     * Would require signals for error handling.
     * Probably fastest
     */
//    KIO::FileCopyJob* job = new KIO::FileCopyJob( src, dest , 420, false, true, false, true );
    if( canceled )
        return false;

    locked = true;
    result = 0;
    KURL::List lsrc;
    lsrc.append( KURL(src) );
    KIO::CopyJob* job = new KIO::CopyJob( lsrc, KURL(dest),
                        KIO::CopyJob::Copy, true, false );

    connect( job, SIGNAL( result( KIO::Job * ) ),
                         this, SLOT( copyFinished() ) );
    connect( job, SIGNAL( canceled( KIO::Job * ) ),
                         this, SLOT( cancelJob() ) );

    while( locked ) {
        kapp->processEvents();
    };

    if( result && !canceled )
        return true;

    return false;
}

QString FileOperation::getName( QString file )
{
    QFileInfo info( file );
    return info.fileName();
}

void FileOperation::copyFinished()
{
    result = job->error();
    locked = false;
}

void FileOperation::cancelJob()
{
    result = job->error();
    locked = false;
    canceled = true;
}
