/***************************************************************************
                          fileplugin.cpp  -  description
                             -------------------
    begin                : Mon Jul 1 2002
    copyright            : (C) 2002 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fileplugin.h"

// Qt includes
#include <qcheckbox.h>
#include <qlabel.h>
#include <qlayout.h>

// KDE includes
#include <kapplication.h>
#include <kfilemetainfo.h>
#include <klineedit.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <kmimetype.h>

FilePlugin::FilePlugin( KService* service )
{
    buttonWrite = 0;

    KFileMetaInfoProvider* mip = KFileMetaInfoProvider::self();
    m_name = service->name();
    m_comment = service->comment();
    QStringList options = service->serviceTypes();
    for( unsigned int i = 0; i < options.count(); i++ ) {
        if( options[i] != "KFilePlugin" ) {
            m_mimetype = options[i];
            const KFileMimeTypeInfo* info = mip->mimeTypeInfo( m_mimetype );
            if( info )
                keys = info->supportedKeys();

            fileplugin = mip->plugin( m_mimetype );
           
            KMimeType::Ptr mime = KMimeType::mimeType( m_mimetype );
            m_icon = mime->icon( QString::null, true );  // arguments are unused
        }
    }
}

FilePlugin::~FilePlugin()
{ }

const QString FilePlugin::getName() const
{
    return m_name;
}

const QString FilePlugin::getAccelName() const
{
    return "&" + getName();
}

const int FilePlugin::type() const
{
    return TYPE_ALL;
}

bool FilePlugin::checkError()
{
    return true;
}

void FilePlugin::drawInterface( QWidget* w, QVBoxLayout* l )
{
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Expanding );
    
    QHBoxLayout* hbox = new QHBoxLayout( 0, 6, 6 );
    
    QLabel* pix = new QLabel( w );
    pix->setPixmap( kapp->iconLoader()->loadIcon( m_icon, KIcon::Desktop ) );

    buttonWrite = new KPushButton( i18n("&Write Meta-Info to File..."), w );

    hbox->addWidget( pix );
    hbox->addWidget( new QLabel( "<qt><b>"+getName()+"</b></qt>", w ) );
    hbox->addItem( spacer );

    l->addLayout( hbox );    
    l->addWidget( new QLabel( m_comment, w ) );
    QString text = i18n("Supported tokens:");
    l->addWidget( new QLabel( text, w ) );

    KListBox* list = new KListBox( w );
    list->setColumnMode( KListBox::FitToWidth );

    for( unsigned int i = 0; i < keys.count(); i++ )
        list->insertItem( "[" + keys[i] + "]" );

    QHBoxLayout* hb2 = new QHBoxLayout( 0, 6, 6 );
    hb2->addWidget( buttonWrite );
    hb2->addItem( spacer );

    l->addWidget( list );
    l->setStretchFactor( list, 2 );
    l->addLayout( hb2 );
   
    connect( buttonWrite, SIGNAL( clicked() ), this, SLOT( changeMetaInfo() ) );
}

void FilePlugin::fillStructure()
{
}

QString FilePlugin::processFile( values* val, int i, QString token, int mode )
{
    QString filename = val->files[i].source_dir + val->files[i].source_name + ( val->files[i].extension.isEmpty() ? "" : "." ) + val->files[i].extension;
    
    if( mode == TYPE_BRACKET ) {
        /*
         * Check if we have something cached for this file
         */
        if( cache.contains( filename + "::" + token ) )
            return cache[filename + "::" + token ];
    
        for( unsigned int i = 0; i < keys.count(); i++ ) {
            if( token.lower() == keys[i].lower() ) {
                KFileMetaInfo meta( filename );
                if( meta.isValid() ) {
                    QString ret = meta.item( keys[i] ).string( true ).stripWhiteSpace();

                    if( cache.count() >= CACHE_MAX )
                        cache.remove( cache.begin() );

                    cache.insert( filename + "::" + token, ret );

                    kapp->processEvents();
                    return ret;                    
                }
            }
        }
    } else if( mode == TYPE_FINAL_FILE ) {
        for( unsigned int i = 0; i < keys.count(); i++ ) {
            if( map.contains( keys[i] ) ) {
                KFileMetaInfo meta( token );
                if( !meta.isValid() || meta.isEmpty() )
                    continue;

                if(!meta.item( keys[i] ).setValue( map[keys[i]] ))
                    qDebug( "Could not write meta info" );

                if(!meta.applyChanges())
                    qDebug("Aplying failed");
            }
        }

        return QString::null;
    } else if( mode == TYPE_TOKEN ) {
        // even if you do not look for a token
        // return the argument!
        return token;
    }

    return QString::null;
}

void FilePlugin::finished()
{
}

void FilePlugin::addHelp( HelpDialog* combo )
{
    QStringList list;
    for( unsigned int i = 0; i < keys.count(); i++ )
        list.append( "[" + keys[i] + "]" + ";;" + keys[i] );
    
    combo->add( getName(), &list );
}

void FilePlugin::removeHelp( HelpDialog* combo )
{
    combo->remove( getName() );
}

void FilePlugin::setUsed( bool m )
{
    if( buttonWrite )
        buttonWrite->setEnabled( m );
}

const QPixmap FilePlugin::getIcon() const 
{
    return kapp->iconLoader()->loadIcon( m_icon, KIcon::Small );
}

void FilePlugin::changeMetaInfo()
{
    MetaInfoDialog* mid = new MetaInfoDialog( 0 );
    mid->setKeys( &keys, fileplugin, m_mimetype );
    for( unsigned int i = 0; i < keys.count(); i++ )
        if( map.contains( keys[i] ) )
            for( unsigned int z = 0; z < mid->count(); z++ )
                if( (mid->label[z])->text() == keys[i] ) {
                    (mid->check[z])->setChecked( true );
                    (mid->value[z])->setText( map[keys[i]] );
                }
    mid->enableLineEdits();

    if( mid->exec() == QDialog::Accepted ) {
        map.clear();
        for( unsigned int i = 0; i < mid->count(); i++ ) {
            if( mid->value[i]->isEnabled() )
                map.insert( mid->label[i]->text(), mid->value[i]->text() );
        }
    }
}

MetaInfoDialog::MetaInfoDialog( QWidget* parent )
    : KDialogBase( KDialogBase::Plain, "KRename",
      KDialogBase::Ok | KDialogBase::Cancel, KDialogBase::Ok, parent, 0, true, true )
{
    QVBoxLayout* layout = new QVBoxLayout( plainPage(), 6, 6 );

    view = new QScrollView( plainPage() );

    layout->addWidget( view );
}

MetaInfoDialog::~MetaInfoDialog()
{
    delete [] value;
    delete [] check;
    delete [] label;
}

void MetaInfoDialog::setKeys( QStringList* k, KFilePlugin* p, QString mimetype )
{
    QWidget* big_box = new QWidget(view->viewport());
    view->addChild(big_box);

    value = new KLineEdit*[k->count()];
    check = new QCheckBox*[k->count()];
    label = new QLabel*[k->count()];
    
    QVBoxLayout* v = new QVBoxLayout( big_box );
    unsigned int z = 0;
    for( unsigned int i = 0; i < k->count(); i++ ) {
        const QString key = (*k)[i];
        QValidator* val = p->createValidator( mimetype, QString::null, key, 0, key );
        if( !val )
            continue;

        QHBoxLayout* l = new QHBoxLayout( 0, 6, 6 );

        check[z] = new QCheckBox( i18n("Enable "), view->viewport() );
        label[z] = new QLabel( (*k)[i], view->viewport() );
        value[z] = new KLineEdit( view->viewport() );
        value[z]->setValidator( val );
        value[z]->setEnabled( false );

        connect( check[z], SIGNAL( clicked() ), this, SLOT( enableLineEdits() ) );

        l->addWidget( check[z] );
        l->addWidget( label[z] );
        l->addWidget( value[z] );
        
        v->addLayout( l );
        z++;
    }

    m_count = z;
    
    if( !z )
        v->addWidget( new QLabel( i18n("<b>You cannot change meta infos with this plugin.</b>"), big_box ) );
}

void MetaInfoDialog::enableLineEdits()
{
    for( unsigned int i = 0; i < count(); i++ )
        value[i]->setEnabled( check[i]->isChecked() );
}
