/***************************************************************************
                          kmylistbox.cpp  -  description
                             -------------------
    begin                : Tue Oct 16 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// QT includes
#include <qdir.h>
#include <qdragobject.h>
#include <qpainter.h>

// KDE includes
#include <kapplication.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kio/previewjob.h>

// Own includes
#include "kmylistbox.h"

KMyListBox::KMyListBox(QWidget* parent, const char* name, WFlags fl)
    :KListBox(parent, name, fl)
{
    drag = ctrlPressed = shiftPressed = mousePressed = false;
    previewActive = true;
    moving = false;

    errorpix = DesktopIcon( "error" );

    setAcceptDrops(TRUE);
    setSelectionMode(Extended);

    connect( this, SIGNAL(doubleClicked(QListBoxItem*)), this, SLOT(openFile(QListBoxItem*)));
    connect( this, SIGNAL(returnPressed(QListBoxItem*)), this, SLOT(openFile(QListBoxItem*)));
}

KMyListBox::~KMyListBox()
{ }

void KMyListBox::setPreviewSize( int size )
{
    previewSize = size;
}

void KMyListBox::addFile( QString filename )
{
    if(!isFile( filename )) {
//        qDebug(filename + " is no file!" );
        return;
    }

    // Is the file already in our list ?
    if( isInList( filename ) ) {
        notadded.append( filename );
        return;
    }

    if( previewActive && !nameActive )
        insertItem( QPixmap(), filename, -1 );
    else
        QListBox::insertItem( filename, -1 );

    return;
}

bool KMyListBox::isFile( QString f )
{
    QFileInfo fi( f );
    if( !fi.exists() )
        return false;

    KFileItem fileItem( KFileItem::Unknown, KFileItem::Unknown, f);
    if( fileItem.isDir() ) {
        if( f.right(1).compare("/") )
            f.append("/");

        addDir( f, false );
        return false;
    }

    return true;
}

void KMyListBox::addDir( QString dirname, bool recursively, bool update, QString filter, bool dirnames )
{
    static int dircounter = 0;

    if( dirname.isEmpty() )
        return;

    if( dirname.right(1).compare("/") )
            dirname.append("/");

    QDir* d = new QDir( dirname ); // be sure to delete it later
    if( recursively ) {
        d->setFilter( QDir::Dirs | QDir::Hidden );

        if( dirnames )
            addDirName( dirname );

        QStringList list = d->entryList();
        for( unsigned int i = 0; i < list.count(); i++ ) {
            QString tmp = QString( "%1%2" ).arg( dirname ).arg( list[i] );
            if( addThisDir( tmp ) ) {
                addDir( tmp, true, false, filter, dirnames );
                if( dircounter == 50 ) {
                    emit updateCount();
                    kapp->processEvents( 0 );
                    dircounter = 0;
                } else
                    dircounter++;
            }
        }
    }

    d->setFilter( QDir::Files );
    d->setNameFilter( filter );
    QStringList list = d->entryList();
    delete d;
    for ( unsigned int i = 0; i < list.count(); i++ ) {
        QString tmp = QString( "%1%2" ).arg( dirname ).arg( list[i] );
        addFile( tmp );
   }

    if( update ) {
        emit updateCount();
        if( previewActive )
            setPreview( previewActive );   
        KApplication::restoreOverrideCursor();
    }
    return;
}

void KMyListBox::addDirName( QString dirname, bool recursive )
{
    if( recursive ) {
        QDir d( dirname );
        d.setFilter( QDir::Dirs | QDir::Hidden );

        QStringList list = d.entryList();
        for( unsigned int i = 0; i < list.count(); i++ ) {
            QString tmp = QString( "%1/%2" ).arg( dirname ).arg( list[i] );
            if( addThisDir( tmp ) )
                addDirName( tmp, true );
        }
    }

    // Is the dir already in our list ?
    if( isInList( dirname ) ) {
        notadded.append( dirname );
        return;
    }

    QListBox::insertItem( dirname, -1 );
}

void KMyListBox::dropEvent(QDropEvent* e)
{
    if( e->source() != this )
        e->accept(QTextDrag::canDecode(e));

    QStringList list;
    if ( QUriDrag::decodeLocalFiles(e, list) ) {
        for( unsigned int i = 0; i < list.count(); i++ )
            addFile( list[i] );

        emit updateCount();
        setPreview( previewActive );
        emit updatePreview();
    }
}

void KMyListBox::dragEnterEvent(QDragEnterEvent* e)
{
    if( e->source() != this )
        e->accept(QTextDrag::canDecode(e));
}

void KMyListBox::viewportMousePressEvent( QMouseEvent* e )
{
   if( moving )
        move( index( itemAt( e->pos() ) ) );
    else {
        QListBox::contentsMousePressEvent(e);
        QPoint p( e->pos() );
        QListBoxItem *i = itemAt( p );
        if ( i ) {
                presspos = e->pos();
                mousePressed = TRUE;
        }
    }
}

void KMyListBox::viewportMouseMoveEvent( QMouseEvent* e )
{
    if ( mousePressed && ( presspos - e->pos() ).manhattanLength() > KApplication::startDragDistance() ) {
        mousePressed = FALSE;
        QListBoxItem *item = itemAt( presspos );
        if ( item ) {
            QStringList source = text( index( item ) );
            for( int i = 0; i < (signed int)count(); i++ )
                if( isSelected(i) && (i != index(item) ))
                    source.append( text(i) );
            QUriDrag* ud = new QUriDrag(viewport());
            ud->setUnicodeUris( source );
            ud->drag();
        }
    }
}

void KMyListBox::viewportMouseReleaseEvent( QMouseEvent* e )
{
    mousePressed = FALSE;
    QListBoxItem* item = itemAt( e->pos() );
    if( e->button() != RightButton )
        selected( item );
}

void KMyListBox::keyPressEvent( QKeyEvent* e )
{
    /*
     * TODO: Document all this keyboard commands
     */
    if( e->key() == Key_Control )
        ctrlPressed = true;
    else if( e->key() == Key_Shift )
        shiftPressed = true;
    else if( e->key() == Key_Up )
        setCurrentItem( currentItem()-1 );
    else if( e->key() == Key_Down )
        setCurrentItem( currentItem()+1 );
    else if( e->key() == Key_Space )
        select( item(currentItem()) );
    else if( e->key() == Key_Return )
        openFile( item(currentItem()) );
    else if( e->key() == Key_Delete ) {
        if( !isSelected( currentItem() ) ) {
            removeItem( currentItem() );
            return;
        }
        unsigned int i = 0;
        do {
            if(isSelected( i ))
                removeItem( i );
            else
                i++;
        } while( i < count() );
    } else if( e->key() == Key_A )
        for( unsigned int i = 0; i < count(); i++ )
            setSelected( i, true );
    else if( e->key() == Key_M ) {
        moveMode();
        return;
    } else if( e->key() == Key_N )
        for( unsigned int i = 0; i < count(); i++ )
            setSelected( i, false );
    else if( e->key() == Key_End )
        setCurrentItem( count()-1 );
    else if( e->key() == Key_Home )
        setCurrentItem( 0 );
    else
        e->ignore();

    emit updateCount();
    setPreview( previewActive );
    emit updatePreview();
}

void KMyListBox::keyReleaseEvent( QKeyEvent* e )
{
    if( e->key() == Key_Control )
        ctrlPressed = false;
    else if( e->key() == Key_Shift )
        shiftPressed = false;
}

void KMyListBox::openFile( QListBoxItem* item )
{
    if( item ) {
        KFileItem* fileItem = new KFileItem( KFileItem::Unknown, KFileItem::Unknown, item->text() );
        fileItem->run();
        delete fileItem;
    }
}

void KMyListBox::moveMode()
{
    if ( !moving ) {
        moving = true;
        KApplication::setOverrideCursor( Qt::sizeAllCursor );
    }
}

void KMyListBox::select( QListBoxItem* item )
{
    if( !ctrlPressed && !shiftPressed )
        /* Single click on the list box,
         * make all items but the clicked
         * one not selected */
        for( int i = 0; i < (signed int)count(); i++ )
            if( i != index(item) )
                setSelected( i, false );

    if( shiftPressed ) {
        if( currentItem() == -1 ) {
            setSelected( item, !isSelected( item ));
            setCurrentItem( item );
            return;
        }
        if( currentItem() > index(item) ) {
            for( int i = index(item); i <= currentItem(); i++ )
                setSelected( i, !isSelected( i ));
            setCurrentItem( item );
        } else if( currentItem() < index(item) ) { /* Works !*/
            for( int i = currentItem()+1; i <= index(item); i++ )
                setSelected( i, !isSelected( i ));
        } else /* c == index(item) */ /* Works !*/
            setSelected( item, !isSelected( item ));
    } else {
        setSelected( item, !isSelected( item ));
        setCurrentItem( item );
    }
}

bool KMyListBox::addThisDir( QString d )
{
    KFileItem fileItem( KFileItem::Unknown, KFileItem::Unknown, d );
    if (!fileItem.isDir())
        return false;

    if( !hidden )
        if(!fileItem.text().left(1).compare( "." ) )
            return false;

    if( fileItem.text() == "." || fileItem.text() == ".." )
        return false;

    return true;
}

void KMyListBox::preview( KURL::List list )
{
    KIO::PreviewJob* job = KIO::filePreview( list, previewSize, previewSize, 0, 100, false, true, 0 );
    connect( job, SIGNAL( gotPreview( const KFileItem*, const QPixmap &) ), this, SLOT( previewDone( const KFileItem*, const QPixmap &) ) );
    connect( job, SIGNAL( failed( const KFileItem* ) ), this, SLOT( previewFailed( const KFileItem* ) ) );
}

void KMyListBox::previewDone( const KFileItem* item, const QPixmap &pixmap )
{
    QString file = item->url().path();
    QListBoxItem* it = findItem( file, Qt::ExactMatch );
    if( it ) {
        int i = index( it );
        if( nameActive )
            QListBox::changeItem( pixmap, text( i ), i );
        else
            changeItem( pixmap, text( i ), i );
    }

    kapp->processOneEvent();
}

void KMyListBox::previewFailed( const KFileItem* item )
{
    if( nameActive )
        return;

    QString file = item->url().path();
    QListBoxItem* it = findItem( file, Qt::ExactMatch );
    if( it ) {
        int i = index( it );
        changeItem( DesktopIcon( "error" ), text( i ), i );
    }

    kapp->processOneEvent();
}

void KMyListBox::setPreview( bool prv )
{
    previewActive = prv;

    int c = currentItem();

    // take care about nameActive!!
    if( previewActive ) {
        KURL::List list;
        for( unsigned int i = 0; i < count(); i++ ) {
            if( !pixmap( i ) || pixmap( i )->isNull() )
                list.append( KURL( text( i ) ) );
            else
                if( !nameActive )
                    changeItem( (*pixmap( i )), text( i ), i );
                else {
                    if( pixmap( i )->serialNumber() == errorpix.serialNumber() )
                        QListBox::changeItem( text( i ), i );
                    else
                        QListBox::changeItem( (*pixmap( i )), text( i ), i );
                }
        }
        preview( list );
    } else
        for( unsigned int i = 0; i < count(); i++ ) {
            if( pixmap( i ) )
                QListBox::changeItem( text(i), i );
        }

    setCurrentItem( c );
}

void KMyListBox::setName( bool name )
{
    nameActive = name;
    setPreview( previewActive );
    if( name ) {
        setColumnMode( FixedNumber );
    } else {
        setColumnMode( FitToWidth );
    }
}

void KMyListBox::move( int i )
{
    KApplication::restoreOverrideCursor();
    moving = false;

    if( count() == 0 )
        return;

    QString tmp;
    QPixmap p;
    unsigned int j = 0;

    QListBox tmpList;
    int nbSelectedBefore = 0;

    for( j = 0 ; j < count() ; j++ ) {
        if( isSelected( j ) ) {
            if( j < (unsigned int)i )
                nbSelectedBefore++;
            tmp = text(j);
            if( previewActive && pixmap( j ) )
                p = (*pixmap( j ));

            removeItem( j );
            j--;
            tmpList.insertItem( p, tmp, tmpList.count() );
        }
    }
    i -= nbSelectedBefore;
    for( j = 0 ; j < tmpList.count() ; j++ ) {
        tmp = tmpList.text(j);
        if( previewActive && tmpList.pixmap( j ) )
            p = (*(tmpList.pixmap( j )));

        if( previewActive )
            if ( nameActive )
                ((QListBox*) this)->insertItem( p, tmp, i );
            else
                this->insertItem( p, tmp, i );
        else
                ((QListBox*) this)->insertItem( tmp, i );
        i++;
        if( (unsigned int)i >= count() )
            i = count();
    }
}

void KMyListBox::moveUp()
{
    if( count() == 0 )
        return;

    unsigned int i = 0;
    QString tmp;
    QPixmap p;
    do {
        if( isSelected( i ) && i ) {
            tmp = text(i);
            if( previewActive && pixmap( i ) )
                p = (*pixmap( i ));

            removeItem( i );

            if( previewActive )
                if ( nameActive )
                    ((QListBox*) this)->insertItem( p, tmp, i-1 );
                else
                    this->insertItem( p, tmp, i-1 );
            else
                ((QListBox*) this)->insertItem( tmp, i-1 );

            setSelected( i-1, true );
            setCurrentItem( i-1 );
        }
        i++;
    } while( i < count());
}

void KMyListBox::moveUp( int i )
{
    if( count() == 0 )
        return;

    QString tmp;
    QPixmap p;

    tmp = text(i);
    if( previewActive && pixmap( i ) )
        p = (*pixmap( i ));

    removeItem( i );

    if( previewActive )
        if( nameActive )
            ((QListBox*)this)->insertItem( p, tmp, i-1 );
        else
            this->insertItem( p, tmp, i-1 );
    else
    ((QListBox*) this)->insertItem( tmp, i-1 );
}

void KMyListBox::moveDown()
{
    if( count() == 0 )
        return;

    unsigned int i = count();
    QString tmp;
    QPixmap p;
    do {
        i--;
        if( isSelected( i ) && (i < count()) ) {
            tmp = text(i);
            if( previewActive && pixmap( i ) )
                p = (*pixmap( i ));

            removeItem( i );
            if( previewActive )
                if( nameActive )
                    QListBox::insertItem( p, tmp, i+1 );
                else
                    insertItem( p, tmp, i+1 );
            else
                QListBox::insertItem( tmp, i+1 );
            setSelected( i+1, true );
            setCurrentItem( i+1 );
        }
    } while( i > 0 );
}

void KMyListBox::moveDown( int i )
{
    if( count() == 0 )
        return;

    QString tmp;
    QPixmap p;

    tmp = text(i);
    if( previewActive && pixmap( i ) )
        p = (*pixmap( i ));

    removeItem( i );
    if( previewActive )
        if( nameActive )
            QListBox::insertItem( p, tmp, i+1 );
        else
            insertItem( p, tmp, i+1 );
    else
        QListBox::insertItem( tmp, i+1 );
}

bool KMyListBox::isInList( QString text ) const
{
    if( findItem( text, Qt::ExactMatch | Qt::CaseSensitive ) )
        return true;

    return false;
}

void KMyListBox::insertItem( const QPixmap &pixmap, const QString &text, int index )
{
    QListBox::insertItem( new KMyListBoxPixmap(pixmap, text), index );
}

void KMyListBox::changeItem( const QPixmap &pixmap, const QString &text, int index )
{
    if( index >= 0 && index < (int)count() )
        QListBox::changeItem( new KMyListBoxPixmap(pixmap, text), index );
}

void KMyListBox::startAdding()
{
    notadded.clear();
    KApplication::setOverrideCursor( Qt::waitCursor );
}

void KMyListBox::endAdding()
{
    KApplication::restoreOverrideCursor();

    if( notadded.count() ) {
#if QT_VERSION >= 0x030100
        KMessageBox::informationList( this, i18n("The following files are already in the list and could not be added:"), notadded );
#else
        KMessageBox::questionYesNoList( this, i18n("The following files are already in the list and could not be added:"), notadded );
#endif
    }

}

void KMyListBox::sortAscending()
{
    sort( true );
}

void KMyListBox::sortDescending()
{
    sort( false );
}

KMyListBoxPixmap::KMyListBoxPixmap( const QPixmap & pix, const QString& text)
    : QListBoxPixmap( pix, text)
{
}

void KMyListBoxPixmap::paint( QPainter *painter )
{
    int itemHeight = height( listBox() );
    int yPos;

    const QPixmap *pm = pixmap();
    if ( pm && ! pm->isNull() ) {
        yPos = ( itemHeight - pm->height() ) / 2;
        painter->drawPixmap( 3, yPos, *pm);
    }

    /*if ( !text().isEmpty() ) {                              s
    QFontMetrics fm = painter->fontMetrics();
    yPos = ( ( itemHeight - fm.height() ) / 2 ) + fm.ascent();
    painter->drawText( pm->width() + 5, yPos, text() );
    }*/
}

int KMyListBoxPixmap::width( const QListBox* ) const
{
    return QMAX( pixmap()->width() + 6, QApplication::globalStrut().width() );
}

