/***************************************************************************
                          kmylistbox.h  -  description
                             -------------------
    begin                : Tue Oct 16 2001
    copyright            : (C) 2001 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMYLISTBOX_H
#define KMYLISTBOX_H

// KDE includes
#include <klistbox.h>
#include <kurl.h>

// Qt includes
#include <qstringlist.h>

class KFileItem;
class QDragObject;
class QPixmap;
class QPoint;
class KMyListBox : public KListBox  {
    Q_OBJECT
    public:
        KMyListBox(QWidget* parent=0, const char* name=0, WFlags fl=0);
        ~KMyListBox();

        // call before you add something
        void startAdding();
        // call after you added something
        void endAdding();
        
        void addFile( QString filename );
        void addDir( QString dirname, bool recursively, bool update = true,
                     QString filter = "*", bool dirnames = false );
        void addDirName( QString dirname, bool recursive = false );
        void setPreview( bool prv );

        void setHiddenDirs( bool b ) {
            hidden = b;
        }
        bool hiddenDirs() const { return hidden; }
        
        void setName( bool name );

        void insertItem ( const QPixmap & pixmap, const QString & text, int index = -1 );
        void changeItem( const QPixmap &pixmap, const QString &text, int index );
        void move( int i );
        void setPreviewSize( int size );
        int getPreviewSize() const { return previewSize; }

    public slots:
        void moveMode();
        void moveDown();
        void moveDown( int i );
        void moveUp();
        void moveUp( int i );

        void sortAscending();
        void sortDescending();

    signals:
        // emited when new item is drag'n'dropped into listobox
        // or when files are removed or something similar
        void updateCount();
        void updatePreview();

    private:
        void preview( KURL::List list );

        void dropEvent( QDropEvent* e );
        void dragEnterEvent( QDragEnterEvent* e );
        void viewportMousePressEvent( QMouseEvent* e );
        void viewportMouseReleaseEvent( QMouseEvent* e );
        void viewportMouseMoveEvent( QMouseEvent* e );
        void keyPressEvent( QKeyEvent* e );
        void keyReleaseEvent( QKeyEvent* e );

        bool isFile( QString f );
        bool isInList( QString text ) const;

        /*
         * Required so that . and .. are ignored when adding
         * recursively. Handles also wether hidden files are
         * added or not.
         */
        bool addThisDir( QString d );

    private slots:
        void openFile( QListBoxItem* item );
        void select( QListBoxItem* item );
        void previewDone( const KFileItem* item, const QPixmap &pixmap );
        void previewFailed( const KFileItem* item );

    protected:

        bool drag;
        bool mousePressed;
        bool ctrlPressed;
        bool shiftPressed;
        bool previewActive;
        bool nameActive;
        bool moving;
        bool hidden;
        int previewSize;
        QPixmap errorpix;

        QDragObject* drobj;
        QPoint presspos;
        QStringList notadded;        
};

class KMyListBoxPixmap : public QListBoxPixmap {
    public:
        KMyListBoxPixmap( const QPixmap &, const QString& );
        virtual void paint( QPainter *painter );
        virtual int width( const QListBox* ) const;
};

#endif
