/***************************************************************************
                          krenameimpl.cpp  -  description
                             -------------------
    begin                : Die Mai 20 2003
    copyright            : (C) 2003 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "krenameimpl.h"

// Own includes
#include "ProgressDialog.h"
#include "confdialog.h"
#include "fileoperation.h"
#include "kmylistview.h"
#include "kmyhistorycombo.h"
#include "pluginloader.h"
#include "replacedialog.h"
#include "undodialog.h"
#include "helpdialog.h"
#include "coorddialog.h"
#include "numberdialog.h"

// KDE includes
#include <kaboutapplication.h>
#include <kapplication.h>
#include <kaction.h>
#include <kcmdlineargs.h>
#include <kdirselectdialog.h>
#include <kfiledialog.h>
#include <kfileitem.h>
#include <khelpmenu.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kmenubar.h>
#include <kpopupmenu.h>
#include <kurlrequester.h>
#include <klocale.h>

// QT includes
#include <qbuttongroup.h>
#include <qcursor.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qradiobutton.h>
#include <qrect.h>
#include <qregexp.h>
#include <qtimer.h>
#include <qtooltip.h>
#include <qvgroupbox.h>
#include <qvbox.h>
#include <qwidget.h>

const QString pageTitle[] = {
    I18N_NOOP( "F&iles" ),
    I18N_NOOP( "Des&tination" ),
    I18N_NOOP( "P&lugins" ),
    I18N_NOOP( "File&name" )
};

extern QWidget* launch( QRect rect, const QStringList & list = 0, KRenameImpl* impl = 0 );

/* A small helper class to allow layout changes in KDirSelectDialog */
class DSDirSelectDialog : public KDirSelectDialog {
    public:
        DSDirSelectDialog( QString lastpath, bool localOnly, QWidget* parent, bool showfilter = false )
            : KDirSelectDialog( lastpath, localOnly, parent )
        {
            QVBox* vbox = new QVBox( mainWidget() );
            m_mainLayout->addWidget( vbox );
            
            if( showfilter ) {
                QHBox* hbox = new QHBox( vbox );

                label = new QLabel( i18n("&Filter:"), hbox );
                mfilter = new KComboBox( true, hbox );

                hbox->setStretchFactor( mfilter, 2 );
                hbox->setSpacing( 2 );
                
                label->setBuddy( mfilter );
                mfilter->setCurrentText( "*" );
            }

            check = new QCheckBox( i18n("Add sub directories &recursively"), vbox );
        }

        bool recursively() const {
            return check->isChecked();
        }

        void setRecursively( bool b ) {
            check->setChecked( b );
        }

        QString filter() const {
            return mfilter->currentText();
        }
        
    private:
        QCheckBox* check;
        KComboBox* mfilter;
        QLabel* label;
};

KRenameImpl::KRenameImpl( QWidget* p, KMenuBar* m, QPushButton* finish )
    : QObject( (QObject*)p ),
      parent( p ), menuBar( m ), finishButton( finish )
{
    // Load Plugins
    plugin = new PluginLoader();
    m_switching = false;
}

KRenameImpl::~KRenameImpl()
{
    delete plugin;
}

bool KRenameImpl::noRealTimePreview = false;
QString KRenameImpl::lastPath = QString::null;

void KRenameImpl::setup()
{
    parent->setCaption( "KRename" ); // "KRename by Dominik Seichter
    setupActions();
    setupPages();
    updateCount();

    connect( buttonAdd, SIGNAL( clicked() ), this, SLOT( addFile() ));
    connect( buttonAddDir, SIGNAL( clicked() ), this, SLOT( addDir() ));
    connect( buttonAddDirName, SIGNAL( clicked() ), this, SLOT( addDirName() ));
    connect( buttonRemoveAll, SIGNAL( clicked()), this, SLOT( clearList() ));
    connect( buttonRemove, SIGNAL( clicked()), this, SLOT( removeFile() ));
    connect( buttonHelp, SIGNAL( clicked()), this, SLOT( showTokenHelp() ));
    connect( finishButton, SIGNAL( clicked()), this, SLOT( start() ));

    connect( checkExtension, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( optionRename, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( optionMove, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( optionCopy, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( optionLink, SIGNAL( clicked()), this, SLOT( enableControls() ));
    connect( checkUndoScript, SIGNAL( clicked()), this, SLOT( enableControls() ));

    connect( fileList, SIGNAL( updateCount()), this, SLOT( updateCount() ));
    connect( fileList, SIGNAL( updatePreview()), this, SLOT( updatePreview() ));
    connect( buttonUp, SIGNAL( clicked()), fileList, SLOT( moveUp() ));
    connect( buttonDown, SIGNAL( clicked()), fileList, SLOT( moveDown() ));
    connect( buttonUp, SIGNAL( clicked()), this, SLOT( updatePreview() ));
    connect( buttonDown, SIGNAL( clicked()), this, SLOT( updatePreview() ));
    connect( buttonUp2, SIGNAL( clicked()), this, SLOT( moveUp() ));
    connect( buttonDown2, SIGNAL( clicked()), this, SLOT( moveDown() ));
    connect( buttonReplace, SIGNAL( clicked()), this, SLOT( replace() ));
    connect( buttonCoord, SIGNAL( clicked()), this, SLOT( getCoordinates() ));

    connect( filename, SIGNAL( textChanged(const QString &)), this, SLOT( updatePreview() ));
    connect( extemplate, SIGNAL( textChanged(const QString &)), this, SLOT( updatePreview() ));
    connect( checkExtension, SIGNAL( clicked()), this, SLOT( updatePreview() ));
    connect( checkPreview, SIGNAL( clicked()), this, SLOT( toggleImagePreview() ));
    connect( preview, SIGNAL( itemChanged() ), this, SLOT( updatePreview() ));
    connect( preview, SIGNAL( removeItem(int) ), this, SLOT( removeFile(int) ));
    connect( preview, SIGNAL( addFile() ), this, SLOT( addFile() ));
    connect( preview, SIGNAL( addDir() ), this, SLOT( addDir() ));
    connect( undorequester, SIGNAL( urlSelected(const QString &)), this, SLOT( changeUndoScriptName() ));
    connect( kapp, SIGNAL( aboutToQuit() ), this, SLOT( saveConfig() ) );
    connect( buttonMove, SIGNAL( clicked()), fileList, SLOT( moveMode() ));
    connect( checkName, SIGNAL( clicked()), this, SLOT( toggleName() ));
    connect( buttonNumber, SIGNAL( clicked()), this, SLOT( changeNumbers() ));

    // Load Configuration
    loadConfig();
    page_1->setEnabled( true );
    KApplication::restoreOverrideCursor();
        
    filename->setText( "$" );   // no i18n() ! ;)
    extemplate->setText( "$" );

    parseCommandline();
}

void KRenameImpl::parseCommandline()
{
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

    QCStringList optlist = args->getOptionList ( "r" );
    for (QCStringList::ConstIterator it=optlist.begin(); it!=optlist.end(); ++it) {
        fileList->startAdding();
        fileList->addDir( *it, true, true, "*", m_dir );
        fileList->endAdding();
    }

    for( int i = 0; i < args->count(); i++) {
        fileList->startAdding();
        fileList->addFile( args->url( i ).path() );
        fileList->endAdding();
    }

    if( !args->isSet( "preview" ) )
        noRealTimePreview = true;
    else
        noRealTimePreview = false;

    QCString templ = args->getOption( "template" );
    if( !templ.isEmpty() )
        filename->setText( templ );

    templ = args->getOption( "extension" );
    if( !templ.isEmpty() ) {
        extemplate->setText( templ );
        checkExtension->setChecked( false );
    }

    templ = args->getOption( "copy" );
    if( !templ.isEmpty() ) {
        urlrequester->setURL( templ );
        optionCopy->setChecked( true );
    }

    templ = args->getOption( "move" );
    if( !templ.isEmpty() ) {
        urlrequester->setURL( templ );
        optionMove->setChecked( true );
    }
        
    QCStringList uselist = args->getOptionList ( "use-plugin" );
    if( !uselist.isEmpty() ) {
        for(unsigned int i = 0; i < uselist.count(); i++ )
            uselist[i] = uselist[i].lower();
    
        QPtrListIterator<PluginLoader::PluginLibrary> it( plugin->libs );
        while ( it.current() ) {
            if( uselist.contains( (*it)->plugin->getName().lower().utf8() ) )
                (*it)->check->setChecked( true );
                
            ++it;
        }

        pluginHelpChanged();
    }
    
    bool startnow = args->isSet( "start" );
    
    // Free some memory        
    args->clear();      

    enableControls();
    updateCount();
    updatePreview();

    if( startnow && fileList->count() ) {
        start();
        // hide the KRename main window after renaming:
        QTimer::singleShot( 200, parent, SLOT( hide() ) );
    }
}

void KRenameImpl::setupActions()
{
    KActionCollection* actionCollection = new KActionCollection( this );

    KPopupMenu* mnuExtra = new KPopupMenu( parent );
    KPopupMenu* mnuSettings = new KPopupMenu( parent );
    KHelpMenu* mnuHelp = new KHelpMenu( parent );

    menuBar->insertItem( i18n("E&xtras"), mnuExtra );
    mnuExtra->insertItem( SmallIcon("undo"), i18n("&Undo Old Renaming Action..."), this, SLOT( undo() ) );
    menuBar->insertItem( i18n("&Settings"), mnuSettings );
    menuBar->insertSeparator();
    menuBar->insertItem( i18n("&Help"), mnuHelp->menu() );

    KAction* prefAct = KStdAction::preferences( this, SLOT(preferences()), actionCollection );

    prefAct->plug( mnuSettings );

    connect( mnuHelp, SIGNAL(showAboutApplication()), this, SLOT(about()));
}

void KRenameImpl::setupPages()
{
    setupPage1();
    setupPage2();
    setupPage3();
    setupPage4();

    // Disable dirname ;)
    enableControls();
}

void KRenameImpl::setupPage1()
{
    page_1 = new QWidget( parent );

    pageLayout = new QVBoxLayout( page_1, 11, 6 );
    Layout2 = new QHBoxLayout( 0, 0, 6 );
    Layout3 = new QVBoxLayout( 0, 0, 6 );
    Layout4 = new QHBoxLayout( 0, 0, 6 );
    Layout5 = new QVBoxLayout( 0, 0, 6 );

    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    QSpacerItem* spacer2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );

    fileList = new KMyListBox( page_1 );

    buttonAdd = new KPushButton( page_1 );
    buttonAdd->setText( i18n( "&Add Files..." ) );

    buttonAddDir = new KPushButton( page_1 );
    buttonAddDir->setText( i18n( "Add Direct&ory..." ) );
    
    buttonAddDirName = new KPushButton( page_1 );
    buttonAddDirName->setText( i18n("&Rename Directory...") );

    buttonRemove = new KPushButton( page_1 );
    buttonRemove->setText( i18n( "Re&move Files" ) );

    buttonRemoveAll = new KPushButton( page_1 );
    buttonRemoveAll->setText( i18n( "R&emove All" ) );

    buttonSort = new KPushButton( page_1 );
    buttonSort->setText( i18n("&Sort") );

    KPopupMenu* mnuSort = new KPopupMenu( parent );
    mnuSort->insertItem( i18n("Ascending"), fileList, SLOT( sortAscending() ) );
    mnuSort->insertItem( i18n("Descending"), fileList, SLOT( sortDescending() ) );
    
    buttonSort->setPopup( mnuSort );
    
    checkPreview = new QCheckBox( page_1 );
    checkPreview->setText( i18n("&Preview") );

    labelCount = new QLabel( page_1 );

    buttonUp = new KPushButton( page_1 );
    buttonUp->setPixmap( BarIcon( "1uparrow" ) );

    buttonDown = new KPushButton( page_1 );
    buttonDown->setPixmap( BarIcon( "1downarrow" ) );

    checkName = new QCheckBox( page_1 );
    checkName->setText( i18n("&Display name") );

    buttonMove = new KPushButton( page_1 );
    buttonMove->setPixmap( BarIcon( "move" ) );

    Layout5->addWidget( buttonUp );
    Layout5->addWidget( buttonMove );
    Layout5->addWidget( buttonDown );

    Layout4->addLayout( Layout5 );
    Layout4->addItem( spacer2 );
    
    Layout3->addWidget( buttonAdd );
    Layout3->addWidget( buttonAddDir );
    Layout3->addWidget( buttonAddDirName );
    Layout3->addWidget( buttonRemove );
    Layout3->addWidget( buttonRemoveAll );
    Layout3->addWidget( buttonSort );
    Layout3->addWidget( checkPreview );
    Layout3->addWidget( checkName );
    Layout3->addWidget( labelCount );
    Layout3->addItem( spacer );
    Layout3->addLayout( Layout4 );
    Layout3->addItem( spacer );
    
    Layout2->addWidget( fileList );
    Layout2->setStretchFactor( fileList, 2 );
    Layout2->addLayout( Layout3 );

    pageLayout->addLayout( Layout2 );

    emit pageDone( page_1, i18n( pageTitle[0] ) );

    QToolTip::add( buttonAdd, i18n( "Add one or more files" ) );
    QToolTip::add( buttonAddDir, i18n( "Add all files of a directory. Adding recursively is possible, too." ) );
    QToolTip::add( buttonRemove, i18n( "Remove selected files" ) );
    QToolTip::add( buttonRemoveAll, i18n( "Remove all files" ) );
    QToolTip::add( fileList, i18n( "Drag files or directories into this list to add them.<br>Double click on a file to open it." ) );
    QToolTip::add( checkPreview, i18n( "Enable/disable preview of pictures." ) );
    QToolTip::add( labelCount, i18n( "Displays the number of files in the list." ) );
    QToolTip::add( buttonUp, i18n( "Move selected items up" ));
    QToolTip::add( buttonDown, i18n( "Move selected items down" ));
    QToolTip::add( checkName, i18n( "Enable/disable display of file name." ) );
    QToolTip::add( buttonMove, i18n( "Move selected items (select the new location with the mouse)." ));
}

void KRenameImpl::setupPage2()
{
    page_2 = new QWidget( parent );

    pageLayout_2 = new QVBoxLayout( page_2, 6, 6 );

    QSpacerItem* spacer_3 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );

    groupOptions = new QButtonGroup( page_2 );
    groupOptions->setTitle( i18n( "O&ptions" ) );
    groupOptions->setColumnLayout(0, Qt::Vertical );
    groupOptions->layout()->setSpacing( 6 );
    groupOptions->layout()->setMargin( 11 );
    groupOptionsLayout = new QVBoxLayout( groupOptions->layout() );
    groupOptionsLayout->setAlignment( Qt::AlignTop );

    optionRename = new QRadioButton( groupOptions );
    optionRename->setText( i18n( "&Rename input files" ) );
    optionRename->setChecked( true );

    optionCopy = new QRadioButton( groupOptions );
    optionCopy->setText( i18n( "Cop&y files to destination directory" ) );

    optionMove = new QRadioButton( groupOptions );
    optionMove->setText( i18n( "&Move files to destination directory" ) );

    optionLink = new QRadioButton( groupOptions );
    optionLink->setText( i18n( "Create &symbolic links in destination directory" ) );

    checkOverwrite = new QCheckBox( groupOptions );
    checkOverwrite->setText( i18n( "&Overwrite existing files" ) );

    groupDir = new QGroupBox( page_2 );
    groupDir->setTitle( i18n( "&Destination Directory" ) );
    groupDir->setColumnLayout(0, Qt::Vertical );
    groupDir->layout()->setSpacing( 6 );
    groupDir->layout()->setMargin( 11 );
    groupDirLayout = new QHBoxLayout( groupDir->layout() );
    groupDirLayout->setAlignment( Qt::AlignTop );

    dirname = new KMyHistoryCombo( groupDir );
    urlrequester = new KURLRequester( dirname, groupDir );
    urlrequester->setMode( KFile::Directory | KFile::ExistingOnly );

    groupOptionsLayout->addWidget( optionRename );
    groupOptionsLayout->addWidget( optionCopy );
    groupOptionsLayout->addWidget( optionMove );
    groupOptionsLayout->addWidget( optionLink );
    groupOptionsLayout->addItem( spacer_3 );
    groupOptionsLayout->addWidget( checkOverwrite );

    groupDirLayout->addWidget( urlrequester );

    groupUndo = new QVGroupBox( page_2 );
    groupUndo->setTitle( i18n("&Undo Renaming") );
//    groupUndo->setColumnLayout(0, Qt::Vertical );
//    groupUndo->layout()->setSpacing( 6 );
//    groupUndo->layout()->setMargin( 11 );
//    QVBoxLayout* groupUndoLayout = new QVBoxLayout( groupUndo->layout() );
//    groupUndoLayout->setAlignment( Qt::AlignTop );

    checkUndoScript = new QCheckBox( i18n("&Create an undo script"), groupUndo );

    undorequester = new KURLRequester( groupUndo );
    undorequester->setEnabled( false );
    undorequester->setMode( KFile::File | KFile::LocalOnly  );
    undorequester->setFilter( i18n("*.krename|KRename undo scripts (*.krename)\n"
                                   "*|All Files (*)") );
    undorequester->fileDialog()->setOperationMode( KFileDialog::Saving );

//    groupUndoLayout->addWidget( checkUndoScript );
//    groupUndoLayout->addWidget( undorequester );

    pageLayout_2->addWidget( groupOptions );
    pageLayout_2->addWidget( groupDir );
    pageLayout_2->addWidget( groupUndo );
    pageLayout_2->addItem( spacer_3 );

    emit pageDone( page_2, i18n( pageTitle[1] ) );

    QToolTip::add( optionRename, i18n( "Input files will be renamed." ) );
    QToolTip::add( optionCopy, i18n( "Copies all files to the destination directory and renames them." ) );
    QToolTip::add( optionMove, i18n( "Moves all files to the destination directory and renames them." ) );
    QToolTip::add( optionLink, i18n( "A symbolic link will be created in the destination directory for each file." ) );
}

void KRenameImpl::setupPage3()
{
    page_3 = new QWidget( parent, "page_3" );
    pageLayout_3 = new QVBoxLayout( page_3, 11, 6 );

    janus = new KJanusWidget( page_3, "janus", KJanusWidget::TreeList );
    janus->setShowIconsInTreeList( true );
    janus->setTreeListAutoResize( true );

    setupTab1();

    pageLayout_3->addWidget( janus );

    emit pageDone( page_3, i18n( pageTitle[2] ) );
}

void KRenameImpl::setupPage4()
{
    page_4 = new QWidget( parent );
    pageLayout_4 = new QVBoxLayout( page_4, 11, 6 );

    Layout10 = new QVBoxLayout( 0, 0, 6 );
    Layout13 = new QHBoxLayout( 0, 0, 6 );
    Layout18 = new QHBoxLayout( 0, 0, 6 );
    Layout21 = new QVBoxLayout( 0, 0, 6 );
    Layout22 = new QHBoxLayout( 0, 0, 6 );
    Layout23 = new QVBoxLayout( 0, 0, 6 );

    QSpacerItem* spacer_9 = new QSpacerItem( 200, 20, QSizePolicy::Maximum, QSizePolicy::Maximum );

    labelTemplate = new QLabel( page_4 );
    labelTemplate->setText( i18n( "Template:" ) );

    filename = new KMyHistoryCombo( page_4 );
    filename->setFocus();

    QLabel* labelDot = new QLabel( page_4 );
    labelDot->setText("<b>.</b>");

    extemplate = new KMyHistoryCombo( page_4 );
    extemplate->setEnabled( FALSE );

    buttonHelp = new KPushButton( i18n("&Functions..."), page_4 );
    buttonHelp->setIconSet( SmallIcon("help") );

    checkExtension = new QCheckBox( page_4 );
    checkExtension->setText( i18n( "&Use extension of the input file" ) );
    checkExtension->setChecked( TRUE );

    buttonReplace = new KPushButton( i18n("&Find and Replace..."), page_4 );
    buttonReplace->setIconSet( SmallIconSet("find") );

    buttonNumber = new KPushButton( i18n("&Numbering..."), page_4 );
    buttonCoord = new KPushButton( i18n("&Insert Part of Filename..."), page_4 );

    helpDialog = new HelpDialog( filename->lineEdit() );

    QStringList help;
    help.append("$;;" + i18n("old filename") );
    help.append("%;;" + i18n("old filename converted to lower case") );
    help.append("&;;" + i18n("old filename converted to upper case") );
    help.append("*;;" + i18n("first letter of every word upper case") );
    help.append(";;" + i18n("first letter of filename upper case") );
    help.append("#;;" + i18n("number (try also ##, ###, ... for leading zeros)") );
    help.append("/;;" + i18n("create a sub directory") );
    help.append("\\;;" + i18n("strip whitespaces") );
    help.append("[$x-y];;" + i18n("character x to y of old filename") );
    help.append("[$x;y];;" + i18n("y characters of old filename starting at x") );
    help.append("[$dirname];;" + i18n("insert name of directory") );
    help.append("[$dirname.];;" + i18n("insert name of directories top level directory") );


    helpDialog->add( i18n("Build in Functions:" ), &help, true );
    help.clear();
    help.append("[date];;" + i18n("current date") );
    help.append("[year];;" + i18n("year") );
    help.append("[month];;" + i18n("month") );
    help.append("[day];;" + i18n("day") );
    help.append("[time];;" + i18n("current time") );
    help.append("[hour];;" + i18n("hour") );
    help.append("[minute];;" + i18n("minute") );
    help.append("[second];;" + i18n("second") );
    helpDialog->add( i18n("Date and Time Functions:" ), &help );

    /*
        //TODO: FIX this tooltip
    QToolTip::add( buttonHelp,
    i18n(
        "<b>$</b> old filename<br><b>%</b> old filename"
        " converted to lower case<br><b>&</b> old filename converted to upper case<br>"
        "<b>*</b> first letter of every word uppercase<br>"
        "<b>#</b> Adds a number to the filename (add more #'s for leading zeros)<br>"
        "<b>/</b> creates a subdirectory<br>"
        "<b>[$x-y]</b> Characters x to y of the old filename<br>"
        "<b>[%x-y]</b> Characters x to y converted to lower case"
    ) );
    */
    
    preview = new KMyListView( &changes, fileList, page_4 );
    preview->setSorting( -1 );
    preview->addColumn( i18n( "Origin" ) );
    preview->addColumn( i18n( "Renamed" ) );

    buttonUp2 = new KPushButton( page_4 );
    buttonUp2->setPixmap( BarIcon( "1uparrow" ) );

    buttonDown2 = new KPushButton( page_4 );
    buttonDown2->setPixmap( BarIcon( "1downarrow" ) );

    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Expanding );

    Layout13->addWidget( labelTemplate );
    Layout13->addWidget( filename );
    Layout13->setStretchFactor( filename, 4 );
    Layout13->addWidget( labelDot );
    Layout13->addWidget( extemplate );
    Layout13->addWidget( buttonHelp );

    Layout18->addWidget( buttonReplace );    
    Layout18->addWidget( buttonNumber );
    Layout18->addWidget( buttonCoord );
    Layout18->addItem( spacer_9 );

    Layout23->addItem( spacer );
    Layout23->addWidget( buttonUp2 );
    Layout23->addWidget( buttonDown2 );
    Layout23->addItem( spacer );

    Layout22->addWidget( preview );
    Layout22->addLayout( Layout23 );
    Layout22->setStretchFactor( preview, 2 );

    Layout10->addLayout( Layout13 );
    Layout10->addWidget( checkExtension );
    Layout10->addItem( spacer_9 );
    Layout10->addLayout( Layout18 );
    Layout10->addLayout( Layout22 );

    pageLayout_4->addLayout( Layout10 );
    emit pageDone( page_4, i18n( pageTitle[3] ) );

    QToolTip::add( filename, i18n( "Add a template.<br>Example: <i>picture###</i>" ) );
    QToolTip::add( extemplate, i18n( "Add a template for the file extension.<br>It behaves like the filename template.") );
    QToolTip::add( checkExtension, i18n("Checking this checkbox the same as setting the extension template to $"));
    QToolTip::add( buttonUp2, i18n( "Move selected items up" ));
    QToolTip::add( buttonDown2, i18n( "Move selected items down" ));
    QToolTip::add( preview, i18n( "Double click on an item to modify it." ));
    QToolTip::add( buttonHelp, i18n("Help Dialog with all tokens supported by KRename."));
}

void KRenameImpl::setupTab1()
{
    /*
     * Draw plugin user interfaces
     */

    QFrame* parent;
    QVBoxLayout* Layout;
    QVBoxLayout* gLayout;
    QGroupBox* g;

    QPtrListIterator<PluginLoader::PluginLibrary> it( plugin->libs ); // iterator for dict
    while ( it.current() ) {
        parent =  janus->addPage( (*it)->plugin->getName(), QString::null, (*it)->plugin->getIcon() );
        
        Layout = new QVBoxLayout( parent, 11, 6, "Layout");
        g = new QGroupBox( parent );
        gLayout = new QVBoxLayout( g, 11, 6, "gLayout" );
        
        (*it)->check = new QCheckBox( i18n("&Use this plugin"), parent );

        connect( (*it)->check, SIGNAL( clicked() ), this, SLOT( pluginHelpChanged() ) );

        (*it)->plugin->drawInterface( g, gLayout );
        (*it)->plugin->setUsed( false );

        Layout->addWidget( (*it)->check );
        Layout->addWidget( g );

        ++it;
    }
}

void KRenameImpl::addFile()
{
    QStringList slist( KFileDialog::getOpenFileNames( lastPath, "*", parent ));
    if (!slist.isEmpty() ) {
        fileList->startAdding();
        QStringList::Iterator it = slist.begin();
        lastPath = QFileInfo((*it).latin1()).dirPath( true );
        for ( ; it != slist.end(); ++it )
            fileList->addFile( (*it).latin1() );
        fileList->endAdding();
        fileList->setPreview( checkPreview->isChecked() );
    }
    updateCount();
    updatePreview();
}

void KRenameImpl::addDir()
{
    DSDirSelectDialog* dsd = new DSDirSelectDialog( lastPath, true, parent, true );

    if( dsd->exec() == QDialog::Accepted ) {
        KURL url = dsd->url();
        lastPath = url.path();

        fileList->startAdding();
        if( dsd->recursively() && m_dir )
            fileList->addDirName( lastPath );

        fileList->addDir( lastPath, dsd->recursively(), true, dsd->filter(), m_dir );
        fileList->endAdding();

        updatePreview();
        updateCount();
    }
}

void KRenameImpl::addDirName()
{
    DSDirSelectDialog* dsd = new DSDirSelectDialog( lastPath, true, parent, false );
    
    if( dsd->exec() == QDialog::Accepted ) {
        KURL url = dsd->url();
        lastPath = url.path();

        fileList->startAdding();
        fileList->addDirName( lastPath, dsd->recursively() );
        fileList->endAdding();
        updatePreview();
        updateCount();
    }
}

void KRenameImpl::clearList()
{
    fileList->clear();
    updateCount();
    updatePreview();
}

void KRenameImpl::removeFile()
{
    unsigned int i = 0;
    do {
        if(fileList->isSelected( i ))
            fileList->removeItem( i );
        else
            i++;
    } while( i < fileList->count() );

    updateCount();
    updatePreview();
}

void KRenameImpl::removeFile( int index )
{
    fileList->removeItem( index );
    updateCount();
    updatePreview();
}

void KRenameImpl::help()
{
    kapp->invokeHelp();
}

void KRenameImpl::start()
{
    if(!checkErrors())
        return;

    // Let the plugins check for erorrs
    QPtrListIterator<PluginLoader::PluginLibrary> it( plugin->libs );
    while ( it.current() ) {
        if( (*it)->usePlugin && !(*it)->plugin->checkError() )
            return;
        ++it;
    }

    ProgressDialog* p = new ProgressDialog( 0, "p" );
    p->print(QString( i18n("Starting conversion of %1 files.") ).arg(fileList->count()));

    values* val = new values;
    fillStructures( val );

    /*
     * Structures are filled, save config now!
     */
    filename->addToHistory( filename->text() );
    dirname->addToHistory( dirname->text() );
    extemplate->addToHistory( extemplate->text() );

    // Save History
    KConfig * config = kapp->config();

    config->setGroup("Path");
    config->writeEntry( "History", dirname->historyItems() );
    config->writeEntry( "CompletionItems", dirname->completionObject()->items() );
    config->writeEntry( "CompletionMode", (int) dirname->completionMode() );

    config->setGroup("Template");
    config->writeEntry( "History", filename->historyItems() );
    config->writeEntry( "CompletionItems", filename->completionObject()->items() );
    config->writeEntry( "CompletionMode", (int) filename->completionMode() );

    config->setGroup("ExTemplate");
    config->writeEntry( "History", extemplate->historyItems() );
    config->writeEntry( "CompletionItems", extemplate->completionObject()->items() );
    config->writeEntry( "CompletionMode", (int) extemplate->completionMode() );

    config->sync();

    // save the configuration
    saveConfig();

    b = new BatchRenamer();

    p->show();
    parent->hide();
    b->processFiles( val, p, plugin );

    // Memory leak!!! :  ??
    //delete parent;
}


void KRenameImpl::enableControls()
{
    urlrequester->setEnabled( !optionRename->isChecked() );
    undorequester->setEnabled( checkUndoScript->isChecked() );
    extemplate->setEnabled( !checkExtension->isChecked() );
}

bool KRenameImpl::checkErrors()
{
    if( filename->text().isEmpty() ) {
        KMessageBox::sorry( parent, i18n("Specify a template to use for renaming files.") );
        return false;
    }
    if( dirname->text().isEmpty() && !optionRename->isChecked()) {
        KMessageBox::sorry( parent, i18n("Please give a destination directory !") );
        emit showPage( 2 );
        return false;
    }
    if( !optionRename->isChecked() ) {
        if( dirname->text().right(1).compare("/") )
            dirname->setText(QString("%1/").arg(dirname->text()));

        KFileItem fileItem( KFileItem::Unknown, KFileItem::Unknown, dirname->text());
        if (!fileItem.isDir()) {
            KMessageBox::sorry( parent, i18n("The given destination directory does not exist!") );
            emit showPage( 2 );
            return false;
        }
    }
    if( checkUndoScript->isChecked() && undorequester->url().isEmpty() ) {
        KMessageBox::sorry( parent, i18n("Please give the name of the undo script!") );
        showPage( 2 );
        return false;
    }

    return true;
}

void KRenameImpl::updateCount()
{
    labelCount->setText( QString( i18n("Files: <b>%1</b>") ).arg(fileList->count()));

    buttonCoord->setEnabled( (bool)fileList->count() );
    buttonReplace->setEnabled( (bool)fileList->count() );
    emit enableFinish( (bool)fileList->count() );
    buttonRemove->setEnabled( (bool)fileList->count() );
    buttonRemoveAll->setEnabled( (bool)fileList->count() );
    buttonSort->setEnabled( (bool)fileList->count() );
}

void KRenameImpl::fillStructures( values* v )
{
    QFileInfo fi;
    for( unsigned int i = 0; i < fileList->count(); i++) {
        fi.setFile( fileList->text( i ) );

        data fdata;
        if( !fi.isDir() ) {
            splitFilename( &fi, &fdata.source_name, &fdata.extension );
        } else {
            //fi.isDir() == true//
            fdata.source_name = BatchRenamer::findDirName( "dirname", fileList->text( i ) );
            fdata.extension = "";
        }

        /* Let's do some error checking for files
         * called: file_xy.
         */
        if( fileList->text( i ).right(1) == "." )
            // File ends with a dot
            fdata.source_name.append( "." );

        fdata.source_dir = fi.dirPath( true );
        if( fdata.source_dir.right(1) != "/" )
            fdata.source_dir.append("/");

        v->files.append( fdata );
    }

    v->text = filename->text();
    v->dirname = dirname->text();
    if( checkExtension->isChecked() )
        v->extext = "$";
    else
        v->extext = extemplate->text();

    v->index = m_index;
    v->step = m_step;
    v->overwrite = checkOverwrite->isChecked();

    if( (v->undo = checkUndoScript->isChecked()) )
        v->undoScript = undorequester->url();

    v->skip = skip;
    v->replace = rep;
    v->changes = changes;

    if( optionCopy->isChecked() )
        v->m = COPY;
    else if( optionMove->isChecked() )
        v->m = MOVE;
    else if( optionRename->isChecked() )
        v->m = RENAME;
    else if( optionLink->isChecked() )
        v->m = LINK;

    /*
     * Tell plugins that they should fill their structure, too.
     * Check also which plugins should be used !
     */
    QPtrListIterator<PluginLoader::PluginLibrary> it( plugin->libs );
    for( int z = 0; it.current(); ++it, z++ ) {
        (*it)->usePlugin = (*it)->check->isChecked();
        if( (*it)->usePlugin )
            (*it)->plugin->fillStructure();
    }
}

void KRenameImpl::updatePre()
{
    updatePreview();
    updateCount();
    toggleImagePreview();
}

void KRenameImpl::addFileOrDir( QString name )
{
    QFileInfo fi( name );
    if( fi.isDir() )
        fileList->addDirName( name );
    else
        fileList->addFile( name );
}

void KRenameImpl::updatePreview()
{
    // is there a number in the filename or extension
    if( filename->text().contains( "#" ) || extemplate->text().contains( "#" ) )
        buttonNumber->setEnabled( true );
    else
        buttonNumber->setEnabled( false );

    if( noRealTimePreview )
        return;

    if( filename->isEmpty() && extemplate->isEmpty() && !checkExtension->isChecked() )
        return;

    if( fileList->count() <= 0 ) {
        preview->clear();
        return;
    }

    int h = preview->horizontalScrollBar()->value();
    int v = preview->verticalScrollBar()->value();

    BatchRenamer b;

    values* val = new values;

    fillStructures( val );
    preview->clear();

    KApplication::setOverrideCursor( Qt::WaitCursor );
    b.createPreview( preview, val, plugin );
    KApplication::restoreOverrideCursor();

    preview->horizontalScrollBar()->setValue( h );
    preview->verticalScrollBar()->setValue( v );
}

void KRenameImpl::loadConfig()
{
    KConfig* config = kapp->config();

    config->setGroup("HistSize");
    m_hist = config->readNumEntry("HistoryItems", 10 );
    updateHist();

    config->setGroup("Path");
    QStringList histList = config->readListEntry("History");
    dirname->setHistoryItems( histList );

    QStringList compList = config->readListEntry("CompletionItems");
    if( compList.isEmpty() )
        dirname->completionObject()->setItems( histList );
    else
        dirname->completionObject()->setItems( compList );

    int mode = config->readNumEntry( "CompletionMode", KGlobalSettings::completionMode() );
    dirname->setCompletionMode( (KGlobalSettings::Completion) mode );

    histList.clear();
    compList.clear();
    config->setGroup("Template");
    histList = config->readListEntry("History");
    filename->setHistoryItems( histList );

    compList = config->readListEntry("CompletionItems");
    if( compList.isEmpty() )
        filename->completionObject()->setItems( histList );
    else
        filename->completionObject()->setItems( compList );

    mode = config->readNumEntry( "CompletionMode", KGlobalSettings::completionMode() );
    filename->setCompletionMode( (KGlobalSettings::Completion) mode );

    histList.clear();
    compList.clear();
    config->setGroup("ExTemplate");
    histList = config->readListEntry("History");
    extemplate->setHistoryItems( histList );

    compList = config->readListEntry("CompletionItems");
    if( compList.isEmpty() )
        extemplate->completionObject()->setItems( histList );
    else
        extemplate->completionObject()->setItems( compList );

    mode = config->readNumEntry( "CompletionMode", KGlobalSettings::completionMode() );
    extemplate->setCompletionMode( (KGlobalSettings::Completion) mode );

    config->setGroup("GUISettings");
    // Image Preview
    checkPreview->setChecked(config->readBoolEntry("ImagePreview", false ));
    fileList->setPreview( checkPreview->isChecked() );
    m_index = config->readNumEntry("StartIndex", 0);
    m_step = 1;
    m_ask = config->readBoolEntry("AskOnExit", true );
    m_dir = config->readBoolEntry("AddDirNames", false );
    fileList->setHiddenDirs( config->readBoolEntry("AddHiddenDirs", false ));
    m_loadplugins = config->readBoolEntry("LoadFilePlugins", true );
    m_autosize = config->readBoolEntry("ColumnAutosize", false );
    checkName->setChecked(config->readBoolEntry("Name", false ));
    fileList->setPreviewSize( config->readNumEntry("PreviewSize", 80) );
    fileList->setName( checkName->isChecked() );
    CoordDialog::m_inversion = config->readBoolEntry("Inverse", false);
    m_wizard = config->readBoolEntry( "GUIWizardMode", true );

    refreshColumnMode();
    if( !m_autosize ) {
        preview->setColumnWidth( 0, config->readNumEntry("Column0", config->readNumEntry( "Width", parent->width() ) - 40 /2 - 2 ) );
        preview->setColumnWidth( 1, config->readNumEntry("Column1", config->readNumEntry( "Width", parent->width() ) - 40 /2 - 2 ) );
    }

    if( !checkPreview->isChecked() ) {
        checkName->setChecked( true );
        checkName->setEnabled( false );
    }

    if( config->readBoolEntry("Maximized", false ) ) {
        parent->showMaximized();
    } else {
        parent->resize( config->readNumEntry( "Width", parent->width() ), config->readNumEntry( "Height", parent->height() ) );
    }

    parent->show();
    page_1->setEnabled( false );
    KApplication::setOverrideCursor( Qt::WaitCursor );
    plugin->loadPlugins( m_loadplugins );
    setupTab1();

    config->setGroup("GUISettings");
    // Skip
    skip = config->readIntListEntry( "skip" );

    // Replace Strings
    QStringList find = config->readListEntry( "FindList" );
    QStringList replace = config->readListEntry( "ReplaceList" );
    QValueList<int> regular = config->readIntListEntry( "RegularList" );
    if( !find.isEmpty() && !replace.isEmpty() )
        for( unsigned int i = 0; i < find.count(); i++ ) {
            replacestrings rs;
            rs.find = find[i];
            rs.replace = replace[i];
            rs.reg = regular[i];
            rep.append( rs );
        }

    // File Extension
    config->setGroup("FileExtension");
    fileExtensionMatrix[0] = config->readBoolEntry( "FirstDot", true );
    fileExtensionMatrix[1] = config->readBoolEntry( "LastDot", false );
    fileExtensionMatrix[2] = config->readBoolEntry( "CustomDot", false );
    fileExtensionInt = config->readNumEntry( "CustomDotValue", 0 );
}

void KRenameImpl::saveConfig()
{
    KConfig* config = kapp->config();

    config->setGroup("HistSize");
    config->writeEntry("HistoryItems", m_hist );

    config->setGroup("GUISettings");
    config->writeEntry("firststart", false );
    config->writeEntry("ImagePreview", checkPreview->isChecked() );
    config->writeEntry("StartIndex", m_index );
    config->writeEntry("Maximized", parent->isMaximized() );
    config->writeEntry("Width", parent->width() );
    config->writeEntry("Height", parent->height() );
    config->writeEntry("XPos", parent->x() );
    config->writeEntry("YPos", parent->y() );
    config->writeEntry("AskOnExit", m_ask );
    config->writeEntry("AddDirNames", m_dir );
    config->writeEntry("AddHiddenDirs", fileList->hiddenDirs() );
    config->writeEntry("LoadFilePlugins", m_loadplugins );
    config->writeEntry("ColumnAutosize", m_autosize );
    config->writeEntry("Name", checkName->isChecked() );
    config->writeEntry("PreviewSize", fileList->getPreviewSize() );
    config->writeEntry("Inverse", CoordDialog::m_inversion );
    config->writeEntry("Column0", preview->columnWidth( 0 ) );
    config->writeEntry("Column1", preview->columnWidth( 1 ) );
    config->writeEntry("GUIWizardMode", m_wizard );

    config->setGroup("FileExtension");
    config->writeEntry( "FirstDot", fileExtensionMatrix[0] );
    config->writeEntry( "LastDot", fileExtensionMatrix[1] );
    config->writeEntry( "CustomDot", fileExtensionMatrix[2] );
    config->writeEntry( "CustomDotValue", fileExtensionInt );

    config->sync();
}

void KRenameImpl::replace()
{
    ReplaceDialog* r = new ReplaceDialog( rep, parent );
    if( r->exec() == QDialog::Accepted )
        rep = r->getList();

    updatePreview();
}

void KRenameImpl::toggleImagePreview()
{
    KApplication::setOverrideCursor( Qt::waitCursor );
    fileList->setPreview( checkPreview->isChecked() );
    checkName->setEnabled( checkPreview->isChecked() );
    if( !checkPreview->isChecked() )
        checkName->setChecked( true );
    fileList->setName( checkName->isChecked() );
    KApplication::restoreOverrideCursor();
}

void KRenameImpl::toggleName()
{
    KApplication::setOverrideCursor( Qt::waitCursor );
    fileList->setName( checkName->isChecked() );
    KApplication::restoreOverrideCursor();
}

void KRenameImpl::preferences()
{
    ConfDialog* cd = new ConfDialog( parent, "cd" );
    cd->setLoadPlugins( m_loadplugins );
    cd->setAsk( m_ask );
    cd->setDir( m_dir );
    cd->setHiddenDir( fileList->hiddenDirs() );
    cd->setThumbSize( fileList->getPreviewSize() );
    cd->setAutosize( m_autosize );
    cd->setHistoryItems( m_hist );
    cd->setUseWizard( m_wizard );

    cd->optionFirstDot->setChecked( fileExtensionMatrix[0] );
    cd->optionLastDot->setChecked( fileExtensionMatrix[1] );
    cd->optionCustomDot->setChecked( fileExtensionMatrix[2] );
    cd->spinDot->setValue( fileExtensionInt );

    cd->enableControls();

    if( cd->exec() == QDialog::Accepted ) {
        bool oldwiz = m_wizard;
    
        m_loadplugins = cd->loadplugins();
        m_ask = cd->ask();
        m_dir = cd->dir();
        m_autosize = cd->autosize();
        m_hist = cd->historyItems();
        m_wizard = cd->useWizard();

        refreshColumnMode();

        fileList->setHiddenDirs( cd->hiddendir() );

        if( fileList->getPreviewSize() != cd->thumbSize() ) {
            fileList->setPreviewSize( cd->thumbSize() );
            fileList->setPreview( checkPreview->isChecked() );
        }

        fileExtensionMatrix[0] = cd->optionFirstDot->isChecked();
        fileExtensionMatrix[1] = cd->optionLastDot->isChecked();
        fileExtensionMatrix[2] = cd->optionCustomDot->isChecked();
        fileExtensionInt = cd->spinDot->value();

        updatePreview();
        updateHist();

        if( oldwiz != m_wizard ) {
            /* GUI Mode was changed by the user */
            saveConfig();

            int x = parent->x();
            int y = parent->y();
            int w = parent->width();
            int h = parent->height();

            QWidget* oldparent = parent;
            QWidget* krename = launch( QRect( x, y, w, h ), 0, this );

            m_switching = true;
            oldparent->removeChild( this );
            oldparent->hide();
            krename->insertChild( this );
            krename->show();
            oldparent->close();
            m_switching = false;            
        }
    }
}

void KRenameImpl::moveUp()
{
    QListViewItem* item = preview->selectedItem();
    if(item) {
        int i = preview->itemIndex( item );
        fileList->moveUp( i );
        updatePreview();
        if( (i-1) > 0 ) {
            preview->setCurrentItem( preview->itemAtIndex( i-1 ) );
            preview->ensureItemVisible( preview->itemAtIndex( i-1 ) );
        }
    }
}

void KRenameImpl::moveDown()
{
    QListViewItem* item = preview->selectedItem();
    if(item) {
        int i = preview->itemIndex( item );
        fileList->moveDown( i );
        updatePreview();
        preview->setCurrentItem( preview->itemAtIndex( i+1 ) );
        preview->ensureItemVisible( preview->itemAtIndex( i+1 ) );
    }
}

void KRenameImpl::undo()
{
    UndoDialog* u = new UndoDialog( parent );
    u->exec();
    delete u;
}

void KRenameImpl::changeUndoScriptName()
{
    const char* EXTENSION = ".krename";
    QString script = undorequester->url();
    if( script.right( 8 ) != EXTENSION ) {
        script += EXTENSION;
        undorequester->setURL( script );
    }
}

void KRenameImpl::pluginHelpChanged()
{
    QPtrListIterator<PluginLoader::PluginLibrary> it( plugin->libs ); // iterator for dict
    while ( it.current() ) {
        (*it)->plugin->setUsed( (*it)->check->isChecked() );
        (*it)->plugin->removeHelp( helpDialog );
        if( (*it)->check->isChecked() )
            (*it)->plugin->addHelp( helpDialog );

        ++it;
    }
}


void KRenameImpl::showTokenHelp()
{
    helpDialog->show();
}

void KRenameImpl::getCoordinates()
{
    QString name;
    QListViewItem* item = preview->selectedItem();
    if( item )
        name = item->text( 0 );
    else {
        item = preview->itemAtIndex( 0 );
        if( item )
            name = item->text( 0 );
    }

    QFileInfo fi( name );
    QString extension;
    splitFilename( &fi, &name, &extension );
    CoordDialog* cd = new CoordDialog( name, parent );
    if( cd->exec() == QDialog::Accepted ) {
        QString t = filename->text();

        if( filename->text() == "$" )
            filename->setText( cd->coords() );
        else {
            t.insert( filename->cursorPosition(), cd->coords() );
            filename->setText( t );
        }
    }

    delete cd;
}

void KRenameImpl::splitFilename( QFileInfo* fi, QString* base, QString* extension )
{
    if( fileExtensionMatrix[1] ) {
        *extension = fi->extension( false );
        if( !extension->isEmpty() )
            *base = fi->fileName().left( fi->fileName().length() - extension->length() - 1 );
        else
            *base = fi->baseName();
    } else if( fileExtensionMatrix[2] ) {
        QString file = FileOperation::getName( fi->filePath() );
        int p = 0;

        if( !file.contains( "." ) || !fileExtensionInt || fileExtensionInt > file.contains( "." ) ) {
            *base = file;
            *extension = "";
            return;
        }

        for( int i = 0; i < fileExtensionInt; i++ )
            p = file.find( ".", p + 1 );

        *base = file.left( p );
        *extension = file.right( file.length() - p - 1 );
    } else {
        // fileExtensionMatrix[0] 
        *base = fi->baseName();
        *extension = fi->extension( true );
    }
}

void KRenameImpl::refreshColumnMode()
{
    if( !m_autosize ) {
        preview->setColumnWidthMode( 0, QListView::Manual );
        preview->setColumnWidthMode( 1, QListView::Manual );
    } else {
        preview->setColumnWidthMode( 0, QListView::Maximum );
        preview->setColumnWidthMode( 1, QListView::Maximum );
    }
}

void KRenameImpl::updateHist()
{
    dirname->setMaxCount( m_hist );
    filename->setMaxCount( m_hist );
    extemplate->setMaxCount( m_hist );
}

void KRenameImpl::changeNumbers()
{
    NumberDialog* nd = new NumberDialog( skip, parent );
    nd->spinIndex->setValue( m_index );
    nd->spinStep->setValue( m_step );

    if( nd->exec() ) {
        m_index = nd->spinIndex->value();
        m_step = nd->spinStep->value();
        skip = nd->getList();

        updatePreview();
    }
}


void KRenameImpl::about()
{
    KAboutApplication * d = new KAboutApplication( kapp->aboutData(), parent );
    d->show();
}

void KRenameImpl::changeParent( QWidget* p, KMenuBar* m, QPushButton* finish, QRect r )
{
    parent = p;
    menuBar = m;
    finishButton = finish;

    connect( finishButton, SIGNAL( clicked()), this, SLOT( start() ));

    if( !r.isNull() ) {
        parent->resize( r.size() );
        parent->move( r.x(), r.y() );
    }
        
    setupActions();

    page_1->reparent( parent, QPoint( 0, 0 ) );
    page_2->reparent( parent, QPoint( 0, 0 ) );
    page_3->reparent( parent, QPoint( 0, 0 ) );
    page_4->reparent( parent, QPoint( 0, 0 ) );

    emit pageDone( page_1, i18n( pageTitle[0] ) );
    emit pageDone( page_2, i18n( pageTitle[1] ) );
    emit pageDone( page_3, i18n( pageTitle[2] ) );
    emit pageDone( page_4, i18n( pageTitle[3] ) );
}

