/***************************************************************************
                          pictureplugin.cpp  -  description
                             -------------------
    begin                : Son Apr 14 2002
    copyright            : (C) 2002 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pictureplugin.h"

// QT includes
#include <qimage.h>
#include <qlabel.h>
#include <qlayout.h>

// KDE includes
#include <kapplication.h>
#include <kiconloader.h>
#include <klocale.h>

const QString PicturePlugin::getName() const
{
    return i18n("Picture Plugin");
}

const QString PicturePlugin::getAccelName() const
{
    return i18n("P&icture Plugin");
}

const int PicturePlugin::type() const
{
    return TYPE_BRACKET;
}

bool PicturePlugin::checkError()
{
    return true;
}

const QPixmap PicturePlugin::getIcon() const
{
    return kapp->iconLoader()->loadIcon( "image", KIcon::Small );
}

void PicturePlugin::drawInterface( QWidget* w, QVBoxLayout* l )
{
    QHBoxLayout* hb = new QHBoxLayout( 0, 0, 6 );
    QLabel* la = new QLabel( w );
    la->setText( i18n("<b>Picture Plugin</b>") );
    l->addWidget( la );

    la = new QLabel( w );
    la->setText( i18n( "<qt>Rename pictures using information like their resolution or color mode.</qt>") );
    l->addWidget( la );

    la = new QLabel( w );
    la->setText( i18n("<qt>This plugin supports the following tokens:</qt>") );
    l->addWidget( la );

    la = new QLabel( w );
    la->setText(i18n("<qt><b>[resolution]</b> insert resolution<br>"
                    "<b>[xres]</b> insert x resolution<br></qt>" ) );
    hb->addWidget( la );

    la = new QLabel( w );
    la->setText(i18n("<qt><b>[yres]</b> insert y resolution<br>"
                    "<b>[bitdepth]</b> insert bitdepth<br></qt>" ) );
    hb->addWidget( la );
    l->addLayout( hb );

    QStringList list = QImage::inputFormatList();
    QString tmp = "";
    for( unsigned int i = 0; i < list.count(); i++ ) {
        if( i < list.count() && i )
            tmp += ", ";

        if( i % 8 == 0 && i < list.count() )
            tmp += "<br>";
            
        tmp += list[i];
    }

    la = new QLabel( w );
    la->setText( "<qt>" + i18n( "Supported image formats: " ) + tmp + "</qt>" );
    l->addWidget( la );

    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    l->addItem( spacer );
}

void PicturePlugin::fillStructure()
{
    /* Don't care about this */
}

QString PicturePlugin::processFile( values* val, int i, QString token, int )
{
    resolution = xres = yres = bitdepth =  "";
    QString filename = val->files[i].source_dir + val->files[i].source_name + ( val->files[i].extension.isEmpty() ? "" : "." ) + val->files[i].extension;

    token = token.lower();

    // Bad code follows:
    if( token != "resolution" && token != "xres" && token != "yres" && 
        token != "bitdepth" )
        return QString::null;

    QImage* img = new QImage( filename );
    if( !img ) {
        delete img;
        return QString::null;
    }

    resolution = QString( "%1x%2" ).arg(img->width()).arg(img->height());
    xres = QString::number( img->width() );
    yres = QString::number( img->height() );
    bitdepth = QString::number( img->depth() );

    delete img;

    if( token == "resolution" )
        return resolution;
    else if( token == "xres" )
        return xres;
    else if( token == "yres" )
        return yres;
    else if( token == "bitdepth" )
        return bitdepth;

    return QString::null;
}

void PicturePlugin::finished()
{
    // We don't care about this event!
    return;
}

void PicturePlugin::addHelp( HelpDialog* combo )
{
    QStringList list;
    list.append( "[$resolution];;" + i18n( "insert resolution" ) );
    list.append( "[#xres];;" + i18n( "insert x resolution" ) );
    list.append( "[#yres];;" + i18n( "insert y resolution" ) );
    list.append( "[#bitdepth];;" + i18n( "insert bitdepth" ) );

    combo->add( getName(), &list );
}

void PicturePlugin::removeHelp( HelpDialog* combo )
{
    combo->remove( getName() );
}
