/***************************************************************************
                          servicemenuhandler.cpp  -  description
                             -------------------
    begin                : Son Jun 8 2003
    copyright            : (C) 2003 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "servicemenuhandler.h"

// Qt includes
#include <qfile.h>

// KDE includes
#include <kio/job.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kprocess.h>
#include <kstandarddirs.h>

using namespace KIO;

ServicemenuHandler::ServicemenuHandler(QObject *parent, const char *name )
    : QObject(parent,name)
{
}

ServicemenuHandler::~ServicemenuHandler()
{
}

void ServicemenuHandler::updateServiceEntry()
{
    // if there is no entry installed, we have nothing to update
    if( serviceEntryInstalled() ) {
        const QString path = locateLocal( "data", "konqueror/servicemenus/krenameservicemenu.desktop" );
        if( KStandardDirs::exists( path ) ) {
            QFile::remove( path );
            installServiceEntry();
        }
    }
}

void ServicemenuHandler::installServiceEntry()
{
    QString src1 = locate( "data", "krename/krenameservicemenu.desktop" );
    QString dest = locateLocal( "data", "konqueror/servicemenus/" );
    QString src2 = locate( "data", "krename/krename_dir.desktop" );

    KURL::List src;
    src.append( src1 );
    src.append( src2 );

    CopyJob* job = KIO::copy( src, KURL( dest ) );
    connect( job, SIGNAL( result( KIO::Job* ) ), this, SLOT( copied( KIO::Job* ) ) );
}

void ServicemenuHandler::copied( KIO::Job* job )
{
    if( !job->error() )
        KMessageBox::information( 0, i18n("Added KRename to your service menu.") );
    else
        KMessageBox::sorry( 0, i18n("Can't add KRename to your service menu.") );

    emit status( serviceEntryInstalled() );
}

void ServicemenuHandler::removeServiceEntry()
{
    // Remove local version
    QString dest = locateLocal( "data", "konqueror/servicemenus/krenameservicemenu.desktop" );
    QString dest2 = locateLocal( "data", "konqueror/servicemenus/krename_dir.desktop" );

    bool d1 = true;
    bool d2 = true;

    if( KStandardDirs::exists( dest ) )
        d1 = QFile::remove( dest );

    if( KStandardDirs::exists( dest2 ) )
        d2 = QFile::remove( dest2 );

    if( d1 && d2 )
        KMessageBox::information( 0, i18n("KRename was succesfully removed from your service menu!") );
    else
        KMessageBox::sorry( 0, i18n("Can't remove service menu entry!") );

    // Check for a non local version
    // not requored for dest2, because it was first introduced in KRename 2.7.3
    dest = locate( "data", "konqueror/servicemenus/krenameservicemenu.desktop" );
    if( !dest.isEmpty() ) {
        KMessageBox::information( 0, i18n(
            "<qt>KRename found another service menu entry in :<br><br>") + dest +
            i18n("<br><br>KRename requires your super user password to remove it.</qt>") );


        KProcess p;
        p << "kdesu" << "rm" << dest;
        p.start();
    }

    emit status( serviceEntryInstalled() );
}

bool ServicemenuHandler::serviceEntryInstalled()
{
    if( !KStandardDirs::exists(
        locateLocal( "data", "konqueror/servicemenus/krenameservicemenu.desktop" ) )  &&
        locate( "data", "konqueror/servicemenus/krenameservicemenu.desktop" ).isEmpty() )
        return false;

    return true;

}

void ServicemenuHandler::changeService(bool b)
{
    if( b )
        installServiceEntry();
    else
        removeServiceEntry();
}

